@@ -437,6 +437,8 @@ nautilus_gtk_places_view_dispose (GObject *object)
   g_clear_pointer (&view->popup_menu, gtk_widget_unparent);
   g_clear_weak_pointer (&view->row_for_action);
 
+  gtk_widget_dispose_template (GTK_WIDGET (view), NAUTILUS_TYPE_GTK_PLACES_VIEW);
+
   G_OBJECT_CLASS (nautilus_gtk_places_view_parent_class)->dispose (object);
 }
 
@@ -531,7 +533,6 @@ populate_servers (NautilusGtkPlacesView *view)
   GtkWidget *child;
   char **uris;
   gsize num_uris;
-  int i;
 
   server_list = server_list_load (view);
 
@@ -555,7 +556,7 @@ populate_servers (NautilusGtkPlacesView *view)
 
   gtk_list_store_clear (view->completion_store);
 
-  for (i = 0; i < num_uris; i++)
+  for (gsize i = 0; i < num_uris; i++)
     {
       RemoveServerData *data;
       GtkTreeIter iter;
@@ -1564,10 +1565,8 @@ properties_cb (GtkWidget  *widget,
 
   if (file)
     {
-      list = g_list_append (NULL, file);
+      list = &(GList){.data = file};
       nautilus_properties_window_present (list, widget, NULL, NULL, NULL);
-
-      nautilus_file_list_unref (list);
     }
 }
 
@@ -1748,17 +1747,21 @@ _popover_set_pointing_to_widget (GtkPopover *popover,
                                  GtkWidget  *target)
 {
   GtkWidget *parent;
-  double x, y, w, h;
+  double w, h;
+  graphene_point_t computed_point;
 
   parent = gtk_widget_get_parent (GTK_WIDGET (popover));
 
-  if (!gtk_widget_translate_coordinates (target, parent, 0, 0, &x, &y))
-    return;
 
-  w = gtk_widget_get_allocated_width (GTK_WIDGET (target));
-  h = gtk_widget_get_allocated_height (GTK_WIDGET (target));
+  if (!gtk_widget_compute_point (target, parent,
+                                 &GRAPHENE_POINT_INIT (0,0),
+                                 &computed_point))
+    g_return_if_reached ();
 
-  gtk_popover_set_pointing_to (popover, &(GdkRectangle){x, y, w, h});
+  w = gtk_widget_get_width (GTK_WIDGET (target));
+  h = gtk_widget_get_height (GTK_WIDGET (target));
+
+  gtk_popover_set_pointing_to (popover, &(GdkRectangle){computed_point.x, computed_point.y, w, h});
 }
 
 static gboolean
@@ -1771,7 +1774,7 @@ real_popup_menu (GtkWidget *widget,
   GMount *mount;
   GFile *file;
   gboolean is_root, is_network;
-  double x_in_view, y_in_view;
+  graphene_point_t point_in_view;
 
   view = NAUTILUS_GTK_PLACES_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (row), NAUTILUS_TYPE_GTK_PLACES_VIEW));
 
@@ -1808,15 +1811,14 @@ real_popup_menu (GtkWidget *widget,
   if (view->row_for_action)
     g_object_set_data (G_OBJECT (view->row_for_action), "menu", NULL);
 
-  if (x == -1 && y == -1)
+  if ((x == -1 && y == -1) ||
+      !gtk_widget_compute_point (widget, GTK_WIDGET (view),
+                                 &GRAPHENE_POINT_INIT (x,y),
+                                 &point_in_view))
     _popover_set_pointing_to_widget (GTK_POPOVER (view->popup_menu), widget);
   else
-    {
-      gtk_widget_translate_coordinates (widget, GTK_WIDGET (view),
-                                        x, y, &x_in_view, &y_in_view);
-      gtk_popover_set_pointing_to (GTK_POPOVER (view->popup_menu),
-                                   &(GdkRectangle){x_in_view, y_in_view, 0, 0});
-    }
+    gtk_popover_set_pointing_to (GTK_POPOVER (view->popup_menu),
+                                 &(GdkRectangle){point_in_view.x, point_in_view.y, 0, 0});
 
   g_set_weak_pointer (&view->row_for_action, row);
   if (view->row_for_action)
@@ -1989,17 +1991,19 @@ on_address_entry_show_help_pressed (NautilusGtkPlacesView        *view,
                                     GtkEntry             *entry)
 {
   GdkRectangle rect;
-  double x, y;
+  graphene_point_t computed_point;
 
   /* Setup the auxiliary popover's rectangle */
   gtk_entry_get_icon_area (GTK_ENTRY (view->address_entry),
                            GTK_ENTRY_ICON_SECONDARY,
                            &rect);
-  gtk_widget_translate_coordinates (view->address_entry, GTK_WIDGET (view),
-                                    rect.x, rect.y, &x, &y);
+  if (!gtk_widget_compute_point (view->address_entry, GTK_WIDGET (view),
+                                 &GRAPHENE_POINT_INIT (rect.x, rect.y),
+                                 &computed_point))
+    g_return_if_reached ();
 
-  rect.x = x;
-  rect.y = y;
+  rect.x = computed_point.x;
+  rect.y = computed_point.y;
   gtk_popover_set_pointing_to (GTK_POPOVER (view->server_adresses_popover), &rect);
   gtk_widget_set_visible (view->server_adresses_popover, TRUE);
 }
@@ -2090,7 +2094,7 @@ listbox_header_func (GtkListBoxRow *row,
 
   if (!before)
     {
-      text = g_strdup_printf ("<b>%s</b>", row_is_network ? _("Networks") : _("On This Computer"));
+      text = g_strdup_printf ("<b>%s</b>", row_is_network ? _("Networks") : _("On This Device"));
     }
   else
     {
@@ -2099,7 +2103,7 @@ listbox_header_func (GtkListBoxRow *row,
       before_is_network = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (before), "is-network"));
 
       if (before_is_network != row_is_network)
-        text = g_strdup_printf ("<b>%s</b>", row_is_network ? _("Networks") : _("On This Computer"));
+        text = g_strdup_printf ("<b>%s</b>", row_is_network ? _("Networks") : _("On This Device"));
     }
 
   if (text)
