@@ -23,6 +23,7 @@
  *          Cosimo Cecchi <cosimoc@gnome.org>
  *
  */
+#define G_LOG_DOMAIN "nautilus-application"
 
 #include "nautilus-application.h"
 
@@ -38,11 +39,9 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#define DEBUG_FLAG NAUTILUS_DEBUG_APPLICATION
-#include "nautilus-debug.h"
-
 #include "nautilus-bookmark-list.h"
 #include "nautilus-clipboard.h"
+#include "nautilus-date-utilities.h"
 #include "nautilus-dbus-launcher.h"
 #include "nautilus-dbus-manager.h"
 #include "nautilus-directory-private.h"
@@ -53,17 +52,16 @@
 #include "nautilus-freedesktop-dbus.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-info.h"
-#include "nautilus-lib-self-check-functions.h"
 #include "nautilus-module.h"
 #include "nautilus-preferences-window.h"
 #include "nautilus-previewer.h"
-#include "nautilus-profile.h"
 #include "nautilus-progress-persistence-handler.h"
-#include "nautilus-self-check-functions.h"
+#include "nautilus-scheme.h"
 #include "nautilus-shell-search-provider.h"
 #include "nautilus-signaller.h"
 #include "nautilus-tag-manager.h"
 #include "nautilus-tracker-utilities.h"
+#include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-view.h"
 #include "nautilus-window-slot.h"
@@ -88,8 +86,6 @@ typedef struct
     NautilusTagManager *tag_manager;
 
     NautilusDBusLauncher *dbus_launcher;
-
-    guint previewer_selection_id;
 } NautilusApplicationPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (NautilusApplication, nautilus_application, ADW_TYPE_APPLICATION);
@@ -150,8 +146,6 @@ check_required_directories (NautilusApplication *self)
 
     g_assert (NAUTILUS_IS_APPLICATION (self));
 
-    nautilus_profile_start (NULL);
-
     ret = TRUE;
 
     user_directory = nautilus_get_user_directory ();
@@ -208,7 +202,6 @@ check_required_directories (NautilusApplication *self)
 
     g_slist_free (directories);
     g_free (user_directory);
-    nautilus_profile_end (NULL);
 
     return ret;
 }
@@ -252,7 +245,6 @@ nautilus_application_create_window (NautilusApplication *self)
     gint default_height = 0;
 
     g_return_val_if_fail (NAUTILUS_IS_APPLICATION (self), NULL);
-    nautilus_profile_start (NULL);
 
     window = nautilus_window_new ();
 
@@ -280,8 +272,7 @@ nautilus_application_create_window (NautilusApplication *self)
         gtk_widget_add_css_class (GTK_WIDGET (window), "devel");
     }
 
-    DEBUG ("Creating a new navigation window");
-    nautilus_profile_end (NULL);
+    g_debug ("Creating a new navigation window");
 
     return window;
 }
@@ -290,52 +281,40 @@ static NautilusWindowSlot *
 get_window_slot_for_location (NautilusApplication *self,
                               GFile               *location)
 {
-    NautilusApplicationPrivate *priv;
-    NautilusWindowSlot *slot;
-    NautilusWindow *window;
-    NautilusFile *file;
-    GList *l, *sl;
+    NautilusApplicationPrivate *priv = nautilus_application_get_instance_private (self);
+    g_autofree char *uri = g_file_get_uri (location);
+    g_autoptr (NautilusFile) file = nautilus_file_get_existing (location);
+    g_autoptr (GFile) searched_location = NULL;
 
-    priv = nautilus_application_get_instance_private (self);
-    slot = NULL;
-    file = nautilus_file_get (location);
-
-    if (!nautilus_file_is_directory (file) && !nautilus_file_is_other_locations (file) &&
+    if (file != NULL &&
+        !nautilus_file_is_directory (file) &&
+        !nautilus_file_is_other_locations (file) &&
         g_file_has_parent (location, NULL))
     {
-        location = g_file_get_parent (location);
+        searched_location = g_file_get_parent (location);
     }
     else
     {
-        g_object_ref (location);
+        searched_location = g_object_ref (location);
     }
 
-    for (l = priv->windows; l != NULL; l = l->next)
+    for (GList *l = priv->windows; l != NULL; l = l->next)
     {
-        window = l->data;
+        NautilusWindow *window = l->data;
 
-        for (sl = nautilus_window_get_slots (window); sl; sl = sl->next)
+        for (GList *sl = nautilus_window_get_slots (window); sl; sl = sl->next)
         {
-            NautilusWindowSlot *current = sl->data;
-            GFile *slot_location = nautilus_window_slot_get_location (current);
+            NautilusWindowSlot *slot = sl->data;
+            GFile *slot_location = nautilus_window_slot_get_location (slot);
 
-            if (slot_location && g_file_equal (slot_location, location))
+            if (slot_location && g_file_equal (slot_location, searched_location))
             {
-                slot = current;
-                break;
+                return slot;
             }
         }
-
-        if (slot)
-        {
-            break;
-        }
     }
 
-    nautilus_file_unref (file);
-    g_object_unref (location);
-
-    return slot;
+    return NULL;
 }
 
 void
@@ -401,8 +380,7 @@ real_open_location_full (NautilusApplication *self,
 
     new_uri = g_file_get_uri (location);
 
-    DEBUG ("Application opening location, old: %s, new: %s", old_uri, new_uri);
-    nautilus_profile_start ("Application opening location, old: %s, new: %s", old_uri, new_uri);
+    g_debug ("Application opening location, old: %s, new: %s", old_uri, new_uri);
 
     g_free (old_uri);
     g_free (new_uri);
@@ -464,10 +442,7 @@ static NautilusWindow *
 open_window (NautilusApplication *self,
              GFile               *location)
 {
-    NautilusWindow *window;
-
-    nautilus_profile_start (NULL);
-    window = nautilus_application_create_window (self);
+    NautilusWindow *window = nautilus_application_create_window (self);
 
     if (location != NULL)
     {
@@ -482,8 +457,6 @@ open_window (NautilusApplication *self,
         g_object_unref (home);
     }
 
-    nautilus_profile_end (NULL);
-
     return window;
 }
 
@@ -497,8 +470,6 @@ nautilus_application_open_location (NautilusApplication *self,
     NautilusWindowSlot *slot;
     GList *sel_list = NULL;
 
-    nautilus_profile_start (NULL);
-
     if (selection != NULL)
     {
         sel_list = g_list_prepend (sel_list, nautilus_file_get (selection));
@@ -521,13 +492,11 @@ nautilus_application_open_location (NautilusApplication *self,
     {
         nautilus_file_list_free (sel_list);
     }
-
-    nautilus_profile_end (NULL);
 }
 
 /* Note: when launched from command line we do not reach this method
  * since we manually handle the command line parameters in order to
- * parse --version, --check, etc.
+ * parse --version, etc.
  * However this method is called when open () is called via dbus, for
  * instance when gtk_uri_open () is called from outside.
  */
@@ -543,7 +512,7 @@ nautilus_application_open (GApplication  *app,
     GFile *file;
     gint idx;
 
-    DEBUG ("Open called on the GApplication instance; %d files", n_files);
+    g_debug ("Open called on the GApplication instance; %d files", n_files);
 
     /* Open windows at each requested location. */
     for (idx = 0; idx < n_files; idx++)
@@ -583,12 +552,16 @@ nautilus_application_finalize (GObject *object)
 
     g_hash_table_destroy (priv->notifications);
 
+    nautilus_module_teardown ();
+
     g_clear_object (&priv->undo_manager);
 
     g_clear_object (&priv->tag_manager);
 
     g_clear_object (&priv->dbus_launcher);
 
+    nautilus_trash_monitor_clear ();
+
     G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
 
@@ -598,15 +571,6 @@ do_cmdline_sanity_checks (NautilusApplication *self,
 {
     gboolean retval = FALSE;
 
-    if (g_variant_dict_contains (options, "check") &&
-        (g_variant_dict_contains (options, G_OPTION_REMAINING) ||
-         g_variant_dict_contains (options, "quit")))
-    {
-        g_printerr ("%s\n",
-                    _("--check cannot be used with other options."));
-        goto out;
-    }
-
     if (g_variant_dict_contains (options, "quit") &&
         g_variant_dict_contains (options, G_OPTION_REMAINING))
     {
@@ -630,28 +594,6 @@ out:
     return retval;
 }
 
-static int
-do_perform_self_checks (void)
-{
-#ifndef NAUTILUS_OMIT_SELF_CHECK
-    gtk_init ();
-
-    nautilus_profile_start (NULL);
-    /* Run the checks (each twice) for nautilus and libnautilus-private. */
-
-    nautilus_run_self_checks ();
-    nautilus_run_lib_self_checks ();
-    eel_exit_if_self_checks_failed ();
-
-    nautilus_run_self_checks ();
-    nautilus_run_lib_self_checks ();
-    eel_exit_if_self_checks_failed ();
-    nautilus_profile_end (NULL);
-#endif
-
-    return EXIT_SUCCESS;
-}
-
 static void
 nautilus_application_select (NautilusApplication  *self,
                              GFile               **files,
@@ -744,6 +686,21 @@ action_preferences (GSimpleAction *action,
     nautilus_preferences_window_show (gtk_application_get_active_window (application));
 }
 
+static void
+action_search_settings (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
+{
+    GtkApplication *self = user_data;
+    const char *parameters = "('launch-panel', [<('search', [<'locations'>])>], @a{sv} {})";
+
+    nautilus_dbus_launcher_call (nautilus_dbus_launcher_get (),
+                                 NAUTILUS_DBUS_LAUNCHER_SETTINGS,
+                                 "Activate",
+                                 g_variant_new_parsed (parameters),
+                                 gtk_application_get_active_window (self));
+}
+
 static void
 action_about (GSimpleAction *action,
               GVariant      *parameter,
@@ -813,15 +770,16 @@ action_quit (GSimpleAction *action,
     g_list_free (windows);
 }
 
-const static GActionEntry app_entries[] =
+static const GActionEntry app_entries[] =
 {
-    { "new-window", action_new_window, NULL, NULL, NULL },
-    { "clone-window", action_clone_window, NULL, NULL, NULL },
-    { "preferences", action_preferences, NULL, NULL, NULL },
-    { "about", action_about, NULL, NULL, NULL },
-    { "help", action_help, NULL, NULL, NULL },
-    { "quit", action_quit, NULL, NULL, NULL },
-    { "kill", action_kill, NULL, NULL, NULL },
+    { .name = "new-window", .activate = action_new_window },
+    { .name = "clone-window", .activate = action_clone_window },
+    { .name = "preferences", .activate = action_preferences },
+    { .name = "search-settings", .activate = action_search_settings },
+    { .name = "about", .activate = action_about },
+    { .name = "help", .activate = action_help },
+    { .name = "quit", .activate = action_quit },
+    { .name = "kill", .activate = action_kill },
 };
 
 static void
@@ -847,7 +805,7 @@ nautilus_application_activate (GApplication *app)
 {
     GFile **files;
 
-    DEBUG ("Calling activate");
+    g_debug ("Calling activate");
 
     files = g_malloc0 (2 * sizeof (GFile *));
     files[0] = g_file_new_for_path (g_get_home_dir ());
@@ -889,12 +847,12 @@ nautilus_application_handle_file_args (NautilusApplication *self,
                 g_free (cwd);
             }
 
-            if (nautilus_is_search_directory (file))
+            if (g_file_has_uri_scheme (file, SCHEME_SEARCH))
             {
                 g_autofree char *error_string = NULL;
                 error_string = g_strdup_printf (_("“%s” is an internal protocol. "
                                                   "Opening this location directly is not supported."),
-                                                EEL_SEARCH_URI);
+                                                SCHEME_SEARCH ":");
 
                 g_printerr ("%s\n", error_string);
             }
@@ -945,8 +903,6 @@ nautilus_application_command_line (GApplication            *application,
     gint retval = -1;
     GVariantDict *options;
 
-    nautilus_profile_start (NULL);
-
     options = g_application_command_line_get_options_dict (command_line);
 
     if (g_variant_dict_contains (options, "version"))
@@ -963,15 +919,9 @@ nautilus_application_command_line (GApplication            *application,
         goto out;
     }
 
-    if (g_variant_dict_contains (options, "check"))
-    {
-        retval = do_perform_self_checks ();
-        goto out;
-    }
-
     if (g_variant_dict_contains (options, "quit"))
     {
-        DEBUG ("Killing app, as requested");
+        g_debug ("Killing app, as requested");
         g_action_group_activate_action (G_ACTION_GROUP (application),
                                         "kill", NULL);
         goto out;
@@ -980,8 +930,6 @@ nautilus_application_command_line (GApplication            *application,
     retval = nautilus_application_handle_file_args (self, options);
 
 out:
-    nautilus_profile_end (NULL);
-
     return retval;
 }
 
@@ -990,31 +938,43 @@ nautilus_application_init (NautilusApplication *self)
 {
     static const GOptionEntry options[] =
     {
-#ifndef NAUTILUS_OMIT_SELF_CHECK
-        { "check", 'c', 0, G_OPTION_ARG_NONE, NULL,
-          N_("Perform a quick set of self-check tests."), NULL },
-#endif
-        { "version", '\0', 0, G_OPTION_ARG_NONE, NULL,
-          N_("Show the version of the program."), NULL },
-        { "new-window", 'w', 0, G_OPTION_ARG_NONE, NULL,
-          N_("Always open a new window for browsing specified URIs"), NULL },
-        { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL,
-          N_("Quit Nautilus."), NULL },
-        { "select", 's', 0, G_OPTION_ARG_NONE, NULL,
-          N_("Select specified URI in parent folder."), NULL },
+        {
+            "version", '\0', 0, G_OPTION_ARG_NONE, NULL,
+            N_("Show the version of the program."), NULL
+        },
+        {
+            "new-window", 'w', 0, G_OPTION_ARG_NONE, NULL,
+            N_("Always open a new window for browsing specified URIs"), NULL
+        },
+        {
+            "quit", 'q', 0, G_OPTION_ARG_NONE, NULL,
+            N_("Quit Nautilus."), NULL
+        },
+        {
+            "select", 's', 0, G_OPTION_ARG_NONE, NULL,
+            N_("Select specified URI in parent folder."), NULL
+        },
         { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, NULL, N_("[URI…]") },
 
         /* The following are old options which have no effect anymore. We keep
          * them around for compatibility reasons, e.g. not breaking old scripts.
          */
-        { "browser", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
-          NULL, NULL },
-        { "geometry", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, NULL,
-          NULL, NULL },
-        { "no-default-window", 'n', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
-          NULL, NULL },
-        { "no-desktop", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
-          NULL, NULL },
+        {
+            "browser", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+            NULL, NULL
+        },
+        {
+            "geometry", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, NULL,
+            NULL, NULL
+        },
+        {
+            "no-default-window", 'n', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+            NULL, NULL
+        },
+        {
+            "no-desktop", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, NULL,
+            NULL, NULL
+        },
 
         { NULL }
     };
@@ -1155,10 +1115,7 @@ static void
 nautilus_application_startup (GApplication *app)
 {
     NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
-
-    nautilus_profile_start (NULL);
-    priv = nautilus_application_get_instance_private (self);
+    NautilusApplicationPrivate *priv = nautilus_application_get_instance_private (self);
 
     g_application_set_resource_base_path (G_APPLICATION (self), "/org/gnome/nautilus");
 
@@ -1172,10 +1129,11 @@ nautilus_application_startup (GApplication *app)
     /* initialize preferences and create the global GSettings objects */
     nautilus_global_preferences_init ();
 
+    /* initialize data preference watchers */
+    nautilus_date_setup_preferences ();
+
     /* initialize nautilus modules */
-    nautilus_profile_start ("Modules");
     nautilus_module_setup ();
-    nautilus_profile_end ("Modules");
 
     /* attach menu-provider module callback */
     menu_provider_init_callback ();
@@ -1196,8 +1154,6 @@ nautilus_application_startup (GApplication *app)
         nautilus_tag_manager_maybe_migrate_tracker2_data (priv->tag_manager);
     }
 
-    nautilus_profile_end (NULL);
-
     g_signal_connect (self, "shutdown", G_CALLBACK (on_application_shutdown), NULL);
 
     g_signal_connect_object (gtk_icon_theme_get_for_display (gdk_display_get_default ()),
@@ -1222,8 +1178,8 @@ nautilus_application_dbus_register (GApplication     *app,
         return FALSE;
     }
 
-    priv->fdb_manager = nautilus_freedesktop_dbus_new (connection);
-    if (!nautilus_freedesktop_dbus_register (priv->fdb_manager, error))
+    priv->fdb_manager = nautilus_freedesktop_dbus_new ();
+    if (!nautilus_freedesktop_dbus_register (priv->fdb_manager, connection, error))
     {
         return FALSE;
     }
@@ -1234,7 +1190,7 @@ nautilus_application_dbus_register (GApplication     *app,
         return FALSE;
     }
 
-    priv->previewer_selection_id = nautilus_previewer_connect_selection_event (connection);
+    nautilus_previewer_setup ();
 
     return TRUE;
 }
@@ -1266,12 +1222,7 @@ nautilus_application_dbus_unregister (GApplication    *app,
         g_clear_object (&priv->search_provider);
     }
 
-    if (priv->previewer_selection_id != 0)
-    {
-        nautilus_previewer_disconnect_selection_event (connection,
-                                                       priv->previewer_selection_id);
-        priv->previewer_selection_id = 0;
-    }
+    nautilus_previewer_teardown (connection);
 }
 
 static void
