@@ -47,8 +47,8 @@ gboolean              nautilus_bookmark_get_has_custom_name    (NautilusBookmark
 int                   nautilus_bookmark_compare_with           (gconstpointer          a,
 								gconstpointer          b);
 
-void                  nautilus_bookmark_set_scroll_pos         (NautilusBookmark      *bookmark,
-								const char            *uri);
-char *                nautilus_bookmark_get_scroll_pos         (NautilusBookmark      *bookmark);
+void                  nautilus_bookmark_take_selected_uris     (NautilusBookmark      *bookmark,
+								GStrv                  selected_uris);
+GStrv                 nautilus_bookmark_get_selected_uris      (NautilusBookmark      *bookmark);
 
 G_END_DECLS
