@@ -43,7 +43,7 @@ struct _NautilusColumnChooser
     GtkWidget *window_title;
     GtkWidget *banner;
     GtkWidget *use_custom_box;
-    GtkWidget *use_custom_switch;
+    GtkWidget *use_custom_row;
 
     GMenuModel *row_button_menu;
     GActionGroup *action_group;
@@ -102,22 +102,25 @@ list_changed (NautilusColumnChooser *chooser)
     g_signal_emit (chooser, signals[CHANGED], 0, column_order, visible_columns);
 }
 
+/**
+ * @up: TRUE when moving up (decreasing position), FALSE when moving down
+ */
 static void
 move_row (NautilusColumnChooser *chooser,
           GtkListBoxRow         *row,
-          guint                  increment)
+          gboolean               up)
 {
     guint i = gtk_list_box_row_get_index (row);
     g_autoptr (NautilusColumn) column = g_list_model_get_item (chooser->model, i);
     guint n_items = g_list_model_get_n_items (chooser->model);
 
-    if (increment == 0 || i + increment < 0 || i + increment >= n_items)
+    if ((up && i <= 0) || (!up && i >= n_items))
     {
         return;
     }
 
     g_list_store_remove (G_LIST_STORE (chooser->model), i);
-    g_list_store_insert (G_LIST_STORE (chooser->model), i + increment, column);
+    g_list_store_insert (G_LIST_STORE (chooser->model), i + (up ? -1 : 1), column);
 
     list_changed (chooser);
 }
@@ -138,7 +141,7 @@ static void action_move_row_up (GSimpleAction *action,
 
     g_assert (GTK_IS_LIST_BOX_ROW (chooser->row_with_open_menu));
 
-    move_row (chooser, chooser->row_with_open_menu, -1);
+    move_row (chooser, chooser->row_with_open_menu, TRUE);
 }
 
 static void action_move_row_down (GSimpleAction *action,
@@ -149,13 +152,13 @@ static void action_move_row_down (GSimpleAction *action,
 
     g_assert (GTK_IS_LIST_BOX_ROW (chooser->row_with_open_menu));
 
-    move_row (chooser, chooser->row_with_open_menu, 1);
+    move_row (chooser, chooser->row_with_open_menu, FALSE);
 }
 
 const GActionEntry column_chooser_actions[] =
 {
-    { "move-up", action_move_row_up },
-    { "move-down", action_move_row_down }
+    { .name = "move-up", .activate = action_move_row_up },
+    { .name = "move-down", .activate = action_move_row_down }
 };
 
 static void
@@ -292,7 +295,7 @@ on_create_row_menu_cb (GtkMenuButton *menu_button,
 {
     GAction *action_up;
     GAction *action_down;
-    int row_index = gtk_list_box_row_get_index (row);
+    guint row_index = gtk_list_box_row_get_index (row);
     NautilusColumnChooser *chooser = NAUTILUS_COLUMN_CHOOSER (
         gtk_widget_get_ancestor (GTK_WIDGET (row), NAUTILUS_TYPE_COLUMN_CHOOSER));
 
@@ -324,39 +327,35 @@ add_list_box_row (GObject  *item,
     g_autofree char *label = NULL;
     g_autofree char *name = NULL;
     GtkWidget *row;
-    GtkWidget *row_switch;
     GtkWidget *menu_button;
     GtkWidget *drag_image;
     GtkEventController *controller;
 
     g_object_get (column, "label", &label, "name", &name, NULL);
 
-    row = adw_action_row_new ();
-    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), label);
-
     if (g_strcmp0 (name, "name") == 0)
     {
+        row = adw_action_row_new ();
         adw_action_row_add_prefix (ADW_ACTION_ROW (row), gtk_image_new ());
+        adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), label);
         return row;
     }
 
+    row = adw_switch_row_new ();
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), label);
+
     /* Column can be en/disabled, add switch */
     gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), TRUE);
-    row_switch = gtk_switch_new ();
-    g_object_bind_property (column, "visible", row_switch, "active",
+    g_object_bind_property (column, "visible", row, "active",
                             G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-    g_signal_connect (row_switch, "notify::active",
+    g_signal_connect (row, "notify::active",
                       G_CALLBACK (notify_row_switch_cb),
                       chooser);
 
-    gtk_widget_set_halign (row_switch, GTK_ALIGN_END);
-    gtk_widget_set_valign (row_switch, GTK_ALIGN_CENTER);
-    adw_action_row_add_suffix (ADW_ACTION_ROW (row), row_switch);
-    adw_action_row_set_activatable_widget (ADW_ACTION_ROW (row), row_switch);
-
     /* Add move up/down operation menu */
     menu_button = gtk_menu_button_new ();
     gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (menu_button), "view-more-symbolic");
+    gtk_widget_set_tooltip_text (GTK_WIDGET (menu_button), _("View More"));
     gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), chooser->row_button_menu);
     gtk_menu_button_set_create_popup_func (GTK_MENU_BUTTON (menu_button),
                                            (GtkMenuButtonCreatePopupFunc) on_create_row_menu_cb, row, NULL);
@@ -423,7 +422,7 @@ nautilus_column_chooser_close_request (GtkWindow *window)
     g_auto (GStrv) visible_columns = get_column_names (chooser, TRUE);
     gboolean has_custom;
 
-    has_custom = gtk_switch_get_active (GTK_SWITCH (chooser->use_custom_switch));
+    has_custom = adw_switch_row_get_active (ADW_SWITCH_ROW (chooser->use_custom_row));
     if (has_custom)
     {
         nautilus_column_save_metadata (chooser->file, column_order, visible_columns);
@@ -485,7 +484,7 @@ use_default_clicked_callback (GtkWidget *button,
     set_column_order (chooser, default_order);
 
     gtk_widget_set_visible (chooser->use_custom_box, TRUE);
-    gtk_switch_set_active (GTK_SWITCH (chooser->use_custom_switch), FALSE);
+    adw_switch_row_set_active (ADW_SWITCH_ROW (chooser->use_custom_row), FALSE);
     adw_banner_set_revealed (ADW_BANNER (chooser->banner), FALSE);
 
     list_changed (chooser);
@@ -524,7 +523,7 @@ static void
 nautilus_column_chooser_constructed (GObject *object)
 {
     NautilusColumnChooser *chooser;
-    g_autofree gchar *name = NULL;
+    const char *name = NULL;
     g_auto (GStrv) file_visible_columns = NULL;
     g_auto (GStrv) file_column_order = NULL;
     gboolean has_custom_columns;
@@ -547,7 +546,7 @@ nautilus_column_chooser_constructed (GObject *object)
                           (file_column_order != NULL && file_column_order[0] != NULL) ||
                           is_special_folder (chooser));
 
-    gtk_switch_set_active (GTK_SWITCH (chooser->use_custom_switch), has_custom_columns);
+    adw_switch_row_set_active (ADW_SWITCH_ROW (chooser->use_custom_row), has_custom_columns);
     gtk_widget_set_visible (chooser->use_custom_box, !has_custom_columns);
     adw_banner_set_revealed (ADW_BANNER (chooser->banner), has_custom_columns);
 
@@ -557,6 +556,16 @@ nautilus_column_chooser_constructed (GObject *object)
     }
 }
 
+static void
+nautilus_column_chooser_dispose (GObject *object)
+{
+    NautilusColumnChooser *self = NAUTILUS_COLUMN_CHOOSER (object);
+
+    gtk_widget_dispose_template (GTK_WIDGET (self), NAUTILUS_TYPE_COLUMN_CHOOSER);
+
+    G_OBJECT_CLASS (nautilus_column_chooser_parent_class)->dispose (object);
+}
+
 static void
 nautilus_column_chooser_finalize (GObject *object)
 {
@@ -576,6 +585,7 @@ nautilus_column_chooser_class_init (NautilusColumnChooserClass *chooser_class)
 
     oclass->set_property = nautilus_column_chooser_set_property;
     oclass->constructed = nautilus_column_chooser_constructed;
+    oclass->dispose = nautilus_column_chooser_dispose;
     oclass->finalize = nautilus_column_chooser_finalize;
 
     win_class->close_request = nautilus_column_chooser_close_request;
@@ -585,7 +595,7 @@ nautilus_column_chooser_class_init (NautilusColumnChooserClass *chooser_class)
     gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, window_title);
     gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, banner);
     gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, use_custom_box);
-    gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, use_custom_switch);
+    gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, use_custom_row);
     gtk_widget_class_bind_template_callback (widget_class, use_default_clicked_callback);
     gtk_widget_class_bind_template_child (widget_class, NautilusColumnChooser, row_button_menu);
 
