@@ -154,28 +154,26 @@ void                    nautilus_file_invalidate_all_attributes         (Nautilu
 
 /* Basic attributes for file objects. */
 gboolean                nautilus_file_contains_text                     (NautilusFile                   *file);
-char *                  nautilus_file_get_display_name                  (NautilusFile                   *file);
-char *                  nautilus_file_get_edit_name                     (NautilusFile                   *file);
-char *                  nautilus_file_get_name                          (NautilusFile                   *file);
+const char *            nautilus_file_get_display_name                  (NautilusFile                   *file);
+const char *            nautilus_file_get_edit_name                     (NautilusFile                   *file);
+const char *            nautilus_file_get_name                          (NautilusFile                   *file);
 GFile *                 nautilus_file_get_location                      (NautilusFile                   *file);
-char *			 nautilus_file_get_description			 (NautilusFile			 *file);
 char *                  nautilus_file_get_uri                           (NautilusFile                   *file);
 char *                  nautilus_file_get_uri_scheme                    (NautilusFile                   *file);
 NautilusFile *          nautilus_file_get_parent                        (NautilusFile                   *file);
 GFile *                 nautilus_file_get_parent_location               (NautilusFile                   *file);
 char *                  nautilus_file_get_parent_uri                    (NautilusFile                   *file);
 char *                  nautilus_file_get_parent_uri_for_display        (NautilusFile                   *file);
-char *                  nautilus_file_get_thumbnail_path                (NautilusFile                   *file);
+const char *            nautilus_file_get_thumbnail_path                (NautilusFile                   *file);
 gboolean                nautilus_file_can_get_size                      (NautilusFile                   *file);
-goffset                 nautilus_file_get_size                          (NautilusFile                   *file);
+guint64                 nautilus_file_get_size                          (NautilusFile                   *file);
 time_t                  nautilus_file_get_mtime                         (NautilusFile                   *file);
 time_t                  nautilus_file_get_atime                         (NautilusFile                   *file);
 time_t                  nautilus_file_get_btime                         (NautilusFile                   *file);
 time_t                  nautilus_file_get_recency                       (NautilusFile                   *file);
 time_t                  nautilus_file_get_trash_time                    (NautilusFile                   *file);
 GFileType               nautilus_file_get_file_type                     (NautilusFile                   *file);
-char *                  nautilus_file_get_mime_type                     (NautilusFile                   *file);
-char *                  nautilus_file_get_extension                     (NautilusFile                   *file);
+const char *            nautilus_file_get_mime_type                     (NautilusFile                   *file);
 gboolean                nautilus_file_is_mime_type                      (NautilusFile                   *file,
 									 const char                     *mime_type);
 gboolean                nautilus_file_is_launchable                     (NautilusFile                   *file);
@@ -183,11 +181,12 @@ gboolean                nautilus_file_is_symbolic_link                  (Nautilu
 GMount *                nautilus_file_get_mount                         (NautilusFile                   *file);
 char *                  nautilus_file_get_volume_free_space             (NautilusFile                   *file);
 char *                  nautilus_file_get_volume_name                   (NautilusFile                   *file);
-char *                  nautilus_file_get_symbolic_link_target_path     (NautilusFile                   *file);
+const char *            nautilus_file_get_symbolic_link_target_path     (NautilusFile                   *file);
 char *                  nautilus_file_get_symbolic_link_target_uri      (NautilusFile                   *file);
 gboolean                nautilus_file_is_broken_symbolic_link           (NautilusFile                   *file);
 gboolean                nautilus_file_is_executable                     (NautilusFile                   *file);
 gboolean                nautilus_file_is_directory                      (NautilusFile                   *file);
+gboolean                nautilus_file_is_public_share_folder            (NautilusFile                   *file);
 gboolean                nautilus_file_is_regular_file                   (NautilusFile                   *file);
 gboolean                nautilus_file_is_user_special_directory         (NautilusFile                   *file,
 									 GUserDirectory                 special_directory);
@@ -227,9 +226,7 @@ void                    nautilus_file_set_attributes                    (Nautilu
 									 gpointer                        callback_data);
 GFilesystemPreviewType  nautilus_file_get_filesystem_use_preview        (NautilusFile *file);
 
-char *                  nautilus_file_get_filesystem_id                 (NautilusFile                   *file);
-
-char *                  nautilus_file_get_filesystem_type               (NautilusFile                   *file);
+const char *            nautilus_file_get_filesystem_id                 (NautilusFile                   *file);
 
 gboolean                nautilus_file_get_filesystem_remote             (NautilusFile                   *file);
 
@@ -243,10 +240,10 @@ gboolean                nautilus_file_can_get_owner                     (Nautilu
 gboolean                nautilus_file_can_set_owner                     (NautilusFile                   *file);
 gboolean                nautilus_file_can_get_group                     (NautilusFile                   *file);
 gboolean                nautilus_file_can_set_group                     (NautilusFile                   *file);
-const uid_t             nautilus_file_get_uid                           (NautilusFile                   *file);
+uid_t                   nautilus_file_get_uid                           (NautilusFile                   *file);
 char *                  nautilus_file_get_owner_name                    (NautilusFile                   *file);
-const gid_t             nautilus_file_get_gid                           (NautilusFile                   *file);
-char *                  nautilus_file_get_group_name                    (NautilusFile                   *file);
+gid_t                   nautilus_file_get_gid                           (NautilusFile                   *file);
+const char *            nautilus_file_get_group_name                    (NautilusFile                   *file);
 GList *                 nautilus_get_user_names                         (void);
 GList *                 nautilus_get_all_group_names                    (void);
 GList *                 nautilus_file_get_settable_group_names          (NautilusFile                   *file);
@@ -330,6 +327,8 @@ void                    nautilus_file_cancel                            (Nautilu
  */
 gboolean                nautilus_file_is_gone                           (NautilusFile                   *file);
 
+gboolean                nautilus_file_has_been_unmounted                (NautilusFile                   *file);
+
 /* Used in subclasses that handles the rename of a file. This handles the case
  * when the file is gone. If this returns TRUE, simply do nothing
  */
@@ -344,7 +343,7 @@ gboolean                nautilus_file_rename_handle_file_gone           (Nautilu
 gboolean                nautilus_file_is_not_yet_confirmed              (NautilusFile                   *file);
 
 /* Simple getting and setting top-level metadata. */
-char *                  nautilus_file_get_metadata                      (NautilusFile                   *file,
+const char *            nautilus_file_get_metadata                      (NautilusFile                   *file,
 									 const char                     *key,
 									 const char                     *default_metadata);
 gchar **                nautilus_file_get_metadata_list                 (NautilusFile                   *file,
@@ -363,23 +362,7 @@ gboolean                nautilus_file_get_boolean_metadata              (Nautilu
 									 gboolean                        default_metadata);
 void                    nautilus_file_set_boolean_metadata              (NautilusFile                   *file,
 									 const char                     *key,
-									 gboolean                        default_metadata,
 									 gboolean                        metadata);
-int                     nautilus_file_get_integer_metadata              (NautilusFile                   *file,
-									 const char                     *key,
-									 int                             default_metadata);
-void                    nautilus_file_set_integer_metadata              (NautilusFile                   *file,
-									 const char                     *key,
-									 int                             default_metadata,
-									 int                             metadata);
-
-#define UNDEFINED_TIME ((time_t) (-1))
-
-time_t                  nautilus_file_get_time_metadata                 (NautilusFile                  *file,
-									 const char                    *key);
-void                    nautilus_file_set_time_metadata                 (NautilusFile                  *file,
-									 const char                    *key,
-									 time_t                         time);
 
 
 /* Attributes for file objects as user-displayable strings. */
@@ -401,6 +384,7 @@ gboolean                nautilus_file_has_local_path                    (Nautilu
 /* Comparing two file objects for sorting */
 NautilusFileSortType    nautilus_file_get_default_sort_type             (NautilusFile                   *file,
 									 gboolean                       *reversed);
+const char *            nautilus_file_sort_type_get_attribute           (NautilusFileSortType            sort_type);
 
 int                     nautilus_file_compare_for_sort                  (NautilusFile                   *file_1,
 									 NautilusFile                   *file_2,
@@ -465,8 +449,6 @@ gboolean                nautilus_file_is_thumbnailing                   (Nautilu
 /* Convenience functions for dealing with a list of NautilusFile objects that each have a ref.
  * These are just convenient names for functions that work on lists of GtkObject *.
  */
-GList *                 nautilus_file_list_ref                          (GList                          *file_list);
-void                    nautilus_file_list_unref                        (GList                          *file_list);
 void                    nautilus_file_list_free                         (GList                          *file_list);
 GList *                 nautilus_file_list_copy                         (GList                          *file_list);
 GList *			nautilus_file_list_sort_by_display_name		(GList				*file_list);
@@ -484,7 +466,9 @@ GList *                 nautilus_file_list_filter                       (GList
 gboolean                nautilus_file_list_are_all_folders              (const GList                    *files);
 
 /* Debugging */
+
 void                    nautilus_file_dump                              (NautilusFile                   *file);
+void                    nautilus_file_list_debug                        (GList                          *files);
 
 typedef struct NautilusFilePrivate NautilusFilePrivate;
 
@@ -504,10 +488,6 @@ typedef enum {
 	NAUTILUS_DATE_TYPE_RECENCY
 } NautilusDateType;
 
-gboolean                nautilus_file_get_date                          (NautilusFile                   *file,
-                                                                         NautilusDateType                date_type,
-                                                                         time_t                         *date);
-
 typedef struct {
 	GObjectClass parent_slot;
 
@@ -546,9 +526,8 @@ typedef struct {
 							  guint                  *file_count,
 							  guint                  *unreadable_directory_count,
 							  goffset       *total_size);
-	gboolean              (* get_date)               (NautilusFile           *file,
-							  NautilusDateType        type,
-							  time_t                 *date);
+	GDateTime *           (* get_date)               (NautilusFile           *file,
+							  NautilusDateType        type);
 	char *                (* get_where_string)       (NautilusFile           *file);
 
 	void                  (* set_metadata)           (NautilusFile           *file,
