@@ -22,6 +22,9 @@
 #include <gdk/x11/gdkx.h>
 #endif
 
+/* 1 page worth of scroll in 100ms zooms in or out when the ctrl key is held */
+#define SCROLL_TO_ZOOM_INTERVAL 100
+
 /**
  * NautilusListBase:
  *
@@ -53,6 +56,9 @@ struct _NautilusListBasePrivate
     graphene_point_t hover_start_point;
     guint hover_timer_id;
     GtkDropTarget *view_drop_target;
+
+    gdouble amount_scrolled_for_zoom;
+    guint scroll_timeout_id;
 };
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (NautilusListBase, nautilus_list_base, ADW_TYPE_BIN)
@@ -452,7 +458,6 @@ on_item_drag_prepare (GtkDragSource *source,
         }
     }
     selected_files = g_list_reverse (selected_files);
-    file_list = g_slist_reverse (file_list);
 
     gtk_drag_source_set_actions (source, actions);
 
@@ -483,7 +488,6 @@ hover_timer (gpointer user_data)
     NautilusListBase *self = nautilus_view_cell_get_view (cell);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
     g_autoptr (NautilusViewItem) item = nautilus_view_cell_get_item (cell);
-    g_autoptr (GFile) location = NULL;
 
     priv->hover_timer_id = 0;
 
@@ -504,10 +508,11 @@ hover_timer (gpointer user_data)
         return G_SOURCE_REMOVE;
     }
 
-    location = nautilus_file_get_location (file);
-    nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
-                                             location, NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE,
-                                             NULL, NULL, NULL);
+    NautilusViewModel *model = nautilus_list_base_get_model (self);
+    guint i = nautilus_view_cell_get_position (cell);
+
+    gtk_selection_model_select_item (GTK_SELECTION_MODEL (model), i, TRUE);
+    nautilus_list_base_activate_selection (self, FALSE);
 
     return G_SOURCE_REMOVE;
 }
@@ -1042,6 +1047,7 @@ nautilus_list_base_dispose (GObject *object)
     g_clear_object (&priv->directory_as_file);
     g_clear_object (&priv->model);
     g_clear_handle_id (&priv->hover_timer_id, g_source_remove);
+    g_clear_handle_id (&priv->scroll_timeout_id, g_source_remove);
 
     G_OBJECT_CLASS (nautilus_list_base_parent_class)->dispose (object);
 }
@@ -1052,6 +1058,16 @@ nautilus_list_base_finalize (GObject *object)
     G_OBJECT_CLASS (nautilus_list_base_parent_class)->finalize (object);
 }
 
+static void
+on_scroll_timeout (gpointer user_data)
+{
+    NautilusListBase *self = user_data;
+    NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
+
+    priv->scroll_timeout_id = 0;
+    priv->amount_scrolled_for_zoom = 0;
+}
+
 /* handle Ctrl+Scroll, which will cause a zoom-in/out */
 static gboolean
 on_scroll (GtkEventControllerScroll *scroll,
@@ -1060,79 +1076,37 @@ on_scroll (GtkEventControllerScroll *scroll,
            gpointer                  user_data)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (user_data);
+    NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
     GdkModifierType state;
 
     state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (scroll));
+
     if (state & GDK_CONTROL_MASK)
     {
-        if (dy <= -1)
+        if (priv->scroll_timeout_id == 0)
+        {
+            priv->scroll_timeout_id = g_timeout_add_once (SCROLL_TO_ZOOM_INTERVAL,
+                                                          on_scroll_timeout, self);
+        }
+
+        priv->amount_scrolled_for_zoom += dy;
+        if (priv->amount_scrolled_for_zoom <= -1)
         {
             gtk_widget_activate_action (GTK_WIDGET (self), "view.zoom-in", NULL);
-            return GDK_EVENT_STOP;
+            priv->amount_scrolled_for_zoom = 0;
         }
-        else if (dy >= 1)
+        else if (priv->amount_scrolled_for_zoom >= 1)
         {
             gtk_widget_activate_action (GTK_WIDGET (self), "view.zoom-out", NULL);
-            return GDK_EVENT_STOP;
+            priv->amount_scrolled_for_zoom = 0;
         }
+
+        return GDK_EVENT_STOP;
     }
 
     return GDK_EVENT_PROPAGATE;
 }
 
-static void
-on_scroll_begin (GtkEventControllerScroll *scroll,
-                 gpointer                  user_data)
-{
-    GdkModifierType state;
-
-    state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (scroll));
-    if (state & GDK_CONTROL_MASK)
-    {
-        gtk_event_controller_scroll_set_flags (scroll,
-                                               GTK_EVENT_CONTROLLER_SCROLL_VERTICAL |
-                                               GTK_EVENT_CONTROLLER_SCROLL_DISCRETE);
-    }
-}
-
-static void
-on_scroll_end (GtkEventControllerScroll *scroll,
-               gpointer                  user_data)
-{
-    gtk_event_controller_scroll_set_flags (scroll, GTK_EVENT_CONTROLLER_SCROLL_VERTICAL);
-}
-
-static gboolean
-nautilus_list_base_focus (GtkWidget        *widget,
-                          GtkDirectionType  direction)
-{
-    NautilusListBase *self = NAUTILUS_LIST_BASE (widget);
-    NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
-    g_autoptr (GtkBitset) selection = gtk_selection_model_get_selection (GTK_SELECTION_MODEL (priv->model));
-    gboolean no_selection = gtk_bitset_is_empty (selection);
-    gboolean handled;
-
-    handled = GTK_WIDGET_CLASS (nautilus_list_base_parent_class)->focus (widget, direction);
-
-    if (handled && no_selection)
-    {
-        GtkWidget *focus_widget = gtk_root_get_focus (gtk_widget_get_root (widget));
-
-        /* Workaround for https://gitlab.gnome.org/GNOME/nautilus/-/issues/2489
-         * Also ensures an item gets selected when using <Tab> to focus the view.
-         * Ideally to be fixed in GtkListBase instead. */
-        if (focus_widget != NULL)
-        {
-            gtk_widget_activate_action (focus_widget,
-                                        "listitem.select",
-                                        "(bb)",
-                                        FALSE, FALSE);
-        }
-    }
-
-    return handled;
-}
-
 static gboolean
 nautilus_list_base_grab_focus (GtkWidget *widget)
 {
@@ -1225,7 +1199,6 @@ nautilus_list_base_class_init (NautilusListBaseClass *klass)
     object_class->get_property = nautilus_list_base_get_property;
     object_class->set_property = nautilus_list_base_set_property;
 
-    widget_class->focus = nautilus_list_base_focus;
     widget_class->grab_focus = nautilus_list_base_grab_focus;
 
     klass->get_backing_item = default_get_backing_item;
@@ -1298,8 +1271,6 @@ nautilus_list_base_init (NautilusListBase *self)
     gtk_widget_add_controller (priv->scrolled_window, controller);
     gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_CAPTURE);
     g_signal_connect (controller, "scroll", G_CALLBACK (on_scroll), self);
-    g_signal_connect (controller, "scroll-begin", G_CALLBACK (on_scroll_begin), self);
-    g_signal_connect (controller, "scroll-end", G_CALLBACK (on_scroll_end), self);
 
     g_signal_connect_object (nautilus_preferences,
                              "changed::" NAUTILUS_PREFERENCES_CLICK_POLICY,
@@ -1345,12 +1316,6 @@ nautilus_list_base_setup_gestures (NautilusListBase *self)
     g_signal_connect (controller, "pressed",
                       G_CALLBACK (on_view_click_pressed), self);
 
-    controller = GTK_EVENT_CONTROLLER (gtk_gesture_long_press_new ());
-    gtk_widget_add_controller (GTK_WIDGET (self), controller);
-    gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (controller), TRUE);
-    g_signal_connect (controller, "pressed",
-                      G_CALLBACK (on_view_longpress_pressed), self);
-
     /* TODO: Implement GDK_ACTION_ASK */
     drop_target = gtk_drop_target_new (G_TYPE_INVALID, GDK_ACTION_ALL);
     gtk_drop_target_set_preload (drop_target, TRUE);
@@ -1364,6 +1329,18 @@ nautilus_list_base_setup_gestures (NautilusListBase *self)
     priv->view_drop_target = drop_target;
 }
 
+void
+nautilus_list_base_setup_background_longpress (NautilusListBase *self,
+                                               GtkWidget        *child)
+{
+    GtkEventController *controller = GTK_EVENT_CONTROLLER (gtk_gesture_long_press_new ());
+
+    gtk_widget_add_controller (GTK_WIDGET (child), controller);
+    gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (controller), TRUE);
+    g_signal_connect (controller, "pressed",
+                      G_CALLBACK (on_view_longpress_pressed), self);
+}
+
 /**
  * nautilus_list_base_get_backing_item:
  *
