@@ -212,6 +212,7 @@ void
 nautilus_location_entry_set_location (NautilusLocationEntry *entry,
                                       GFile                 *location)
 {
+    g_autofree char *scheme = g_file_get_uri_scheme (location);
     NautilusLocationEntryPrivate *priv;
     gchar *formatted_uri;
 
@@ -223,7 +224,7 @@ nautilus_location_entry_set_location (NautilusLocationEntry *entry,
      * thus should not emit the LOCATION_CHANGED signal. */
     formatted_uri = g_file_get_parse_name (location);
 
-    if (g_file_has_uri_scheme (location, SCHEME_SEARCH))
+    if (nautilus_scheme_is_internal (scheme))
     {
         nautilus_location_entry_set_special_text (entry, "");
     }
@@ -309,7 +310,7 @@ update_completions_store (gpointer callback_data)
     char *completion;
     int i;
     GtkTreeIter iter;
-    int current_dir_strlen;
+    guint current_dir_strlen;
 
     entry = NAUTILUS_LOCATION_ENTRY (callback_data);
     priv = nautilus_location_entry_get_instance_private (entry);
@@ -759,6 +760,9 @@ nautilus_location_entry_init (NautilusLocationEntry *entry)
 
     priv = nautilus_location_entry_get_instance_private (entry);
 
+    gtk_entry_set_input_purpose (GTK_ENTRY (entry), GTK_INPUT_PURPOSE_URL);
+    gtk_entry_set_input_hints (GTK_ENTRY (entry), GTK_INPUT_HINT_NO_SPELLCHECK | GTK_INPUT_HINT_NO_EMOJI);
+
     priv->completer = g_filename_completer_new ();
     g_filename_completer_set_dirs_only (priv->completer, TRUE);
 
