@@ -30,18 +30,11 @@
 #include "nautilus-application.h"
 #include "nautilus-resources.h"
 
-#include "nautilus-debug.h"
-#include <eel/eel-debug.h>
-
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gio/gdesktopappinfo.h>
 
-#include <libxml/parser.h>
-
-#ifdef HAVE_LOCALE_H
 #include <locale.h>
-#endif
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -55,19 +48,23 @@ main (int   argc,
 {
     gint retval;
     NautilusApplication *application;
-
-    if (g_getenv ("NAUTILUS_DEBUG") != NULL)
-    {
-        eel_make_warnings_and_criticals_stop_in_debugger ();
-    }
-
     /* Initialize gettext support */
+    setlocale (LC_ALL, "");
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
 
     g_set_prgname (APPLICATION_ID);
 
+    if (getuid () == 0)
+    {
+        g_warning (_("\n========================================================"
+                     "\nThis app cannot work correctly if run as root (not even"
+                     "\nwith sudo). Consider running `nautilus admin:/` instead."
+                     "\n========================================================"));
+        sleep (7);
+    }
+
     nautilus_register_resource ();
     /* Run the nautilus application. */
     application = nautilus_application_new ();
@@ -83,7 +80,5 @@ main (int   argc,
 
     g_object_unref (application);
 
-    eel_debug_shut_down ();
-
     return retval;
 }
