@@ -132,8 +132,8 @@ set_copy_move_dialog_text (FileConflictDialogData *data)
     time_t source_mtime;
     time_t destination_mtime;
     g_autofree gchar *message = NULL;
-    g_autofree gchar *destination_name = NULL;
-    g_autofree gchar *destination_directory_name = NULL;
+    const char *destination_name;
+    const char *destination_directory_name;
     gboolean source_is_directory;
     gboolean destination_is_directory;
 
@@ -252,7 +252,6 @@ set_file_labels (FileConflictDialogData *data)
     gboolean source_is_directory;
     gboolean destination_is_directory;
     gboolean should_show_type;
-    g_autofree char *destination_mime_type = NULL;
     g_autofree char *destination_date = NULL;
     g_autofree char *destination_size = NULL;
     g_autofree char *destination_type = NULL;
@@ -263,12 +262,11 @@ set_file_labels (FileConflictDialogData *data)
     source_is_directory = nautilus_file_is_directory (data->source);
     destination_is_directory = nautilus_file_is_directory (data->destination);
 
-    destination_mime_type = nautilus_file_get_mime_type (data->destination);
     should_show_type = !nautilus_file_is_mime_type (data->source,
-                                                    destination_mime_type);
+                                                    nautilus_file_get_mime_type (data->destination));
 
     destination_date = nautilus_file_get_string_attribute_with_default (data->destination,
-                                                                        "date_modified_with_time");
+                                                                        "date_modified_full");
     destination_size = nautilus_file_get_string_attribute_with_default (data->destination,
                                                                         "size");
 
@@ -298,7 +296,7 @@ set_file_labels (FileConflictDialogData *data)
     g_string_append_printf (destination_label, "%s %s", _("Last modified:"), destination_date);
 
     source_date = nautilus_file_get_string_attribute_with_default (data->source,
-                                                                   "date_modified_with_time");
+                                                                   "date_modified_full");
     source_size = nautilus_file_get_string_attribute_with_default (data->source,
                                                                    "size");
 
@@ -340,9 +338,7 @@ set_file_labels (FileConflictDialogData *data)
 static void
 set_conflict_and_suggested_names (FileConflictDialogData *data)
 {
-    g_autofree gchar *conflict_name = NULL;
-
-    conflict_name = nautilus_file_get_edit_name (data->destination);
+    const char *conflict_name = nautilus_file_get_edit_name (data->destination);
 
     nautilus_file_conflict_dialog_set_conflict_name (data->dialog,
                                                      conflict_name);
@@ -577,17 +573,15 @@ static gboolean
 open_file_in_application (gpointer user_data)
 {
     HandleUnsupportedFileData *data;
-    g_autofree gchar *mime_type = NULL;
     GtkWidget *dialog;
     const char *heading;
 
     data = user_data;
-    mime_type = nautilus_file_get_mime_type (data->file);
     dialog = gtk_app_chooser_dialog_new_for_content_type (data->parent_window,
                                                           GTK_DIALOG_MODAL |
                                                           GTK_DIALOG_DESTROY_WITH_PARENT |
                                                           GTK_DIALOG_USE_HEADER_BAR,
-                                                          mime_type);
+                                                          nautilus_file_get_mime_type (data->file));
     heading = _("Password-protected archives are not yet supported. "
                 "This list contains apps that can open the archive.");
 
@@ -625,24 +619,22 @@ typedef struct
     ContextInvokeData parent_type;
     GtkWindow *parent_window;
     const gchar *basename;
-    GtkEntry *passphrase_entry;
+    GtkPasswordEntry *passphrase_entry;
     gchar *passphrase;
 } PassphraseRequestData;
 
 static void
-on_request_passphrase_cb (GtkDialog *dialog,
-                          gint       response_id,
-                          gpointer   user_data)
+on_request_passphrase_cb (AdwMessageDialog *dialog,
+                          gchar            *response,
+                          gpointer          user_data)
 {
     PassphraseRequestData *data = user_data;
 
-    if (response_id != GTK_RESPONSE_CANCEL &&
-        response_id != GTK_RESPONSE_DELETE_EVENT)
+    if (g_str_equal (response, "extract"))
     {
         data->passphrase = g_strdup (gtk_editable_get_text (GTK_EDITABLE (data->passphrase_entry)));
     }
 
-    gtk_window_destroy (GTK_WINDOW (dialog));
     invoke_main_context_completed (data);
 }
 
@@ -653,15 +645,14 @@ run_passphrase_dialog (gpointer user_data)
     g_autofree gchar *label_str = NULL;
     g_autoptr (GtkBuilder) builder = NULL;
     GObject *dialog;
-    GObject *label;
 
     builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/ui/nautilus-operations-ui-manager-request-passphrase.ui");
     dialog = gtk_builder_get_object (builder, "request_passphrase_dialog");
-    label = gtk_builder_get_object (builder, "label");
-    data->passphrase_entry = GTK_ENTRY (gtk_builder_get_object (builder, "entry"));
+    data->passphrase_entry = GTK_PASSWORD_ENTRY (gtk_builder_get_object (builder, "entry"));
 
-    label_str = g_strdup_printf (_("“%s” is password-protected."), data->basename);
-    gtk_label_set_text (GTK_LABEL (label), label_str);
+    adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog),
+                                    _("“%s” is password-protected."),
+                                    data->basename);
 
     g_signal_connect (dialog, "response", G_CALLBACK (on_request_passphrase_cb), data);
     gtk_window_set_transient_for (GTK_WINDOW (dialog), data->parent_window);
