@@ -33,6 +33,7 @@
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-names.h"
+#include "nautilus-scheme.h"
 #include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-window.h"
@@ -139,9 +140,9 @@ static void nautilus_path_bar_clear_buttons (NautilusPathBar *self);
 
 const GActionEntry path_bar_actions[] =
 {
-    { "open-item-new-tab", action_pathbar_open_item_new_tab },
-    { "open-item-new-window", action_pathbar_open_item_new_window },
-    { "properties", action_pathbar_properties}
+    { .name = "open-item-new-tab", .activate = action_pathbar_open_item_new_tab },
+    { .name = "open-item-new-window", .activate = action_pathbar_open_item_new_window },
+    { .name = "properties", .activate = action_pathbar_properties}
 };
 
 
@@ -612,7 +613,7 @@ on_click_gesture_pressed (GtkGestureClick *gesture,
     NautilusPathBar *self;
     guint current_button;
     GdkModifierType state;
-    double x_in_pathbar, y_in_pathbar;
+    graphene_point_t pathbar_point;
 
     if (n_press != 1)
     {
@@ -624,10 +625,6 @@ on_click_gesture_pressed (GtkGestureClick *gesture,
     current_button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
     state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (gesture));
 
-    gtk_widget_translate_coordinates (GTK_WIDGET (button_data->button),
-                                      GTK_WIDGET (self),
-                                      x, y,
-                                      &x_in_pathbar, &y_in_pathbar);
 
     switch (current_button)
     {
@@ -650,8 +647,16 @@ on_click_gesture_pressed (GtkGestureClick *gesture,
             }
             else
             {
+                if (!gtk_widget_compute_point (GTK_WIDGET (button_data->button),
+                                               GTK_WIDGET (self),
+                                               &GRAPHENE_POINT_INIT (x, y),
+                                               &pathbar_point))
+                {
+                    g_return_if_reached ();
+                }
+
                 gtk_popover_set_pointing_to (GTK_POPOVER (self->button_menu_popover),
-                                             &(GdkRectangle){x_in_pathbar, y_in_pathbar, 0, 0});
+                                             &(GdkRectangle){pathbar_point.x, pathbar_point.y, 0, 0});
                 pop_up_pathbar_context_menu (self, button_data->file);
             }
         }
@@ -771,6 +776,10 @@ on_drag_motion (GtkDropTarget *target,
             action = nautilus_dnd_get_preferred_action (button_data->file, items->data);
         }
     }
+    else if (G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE))
+    {
+        action = GDK_ACTION_COPY;
+    }
 
     start = button_data->hover_start_point;
     if (gtk_drag_check_threshold (button_data->button, start.x, start.y, x, y))
@@ -970,7 +979,6 @@ setup_button_type (ButtonData      *button_data,
                    GFile           *location)
 {
     g_autoptr (GMount) mount = NULL;
-    g_autofree gchar *uri = NULL;
 
     if (nautilus_is_root_directory (location))
     {
@@ -981,12 +989,12 @@ setup_button_type (ButtonData      *button_data,
         button_data->type = HOME_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (nautilus_is_recent_directory (location))
+    else if (g_file_has_uri_scheme (location, SCHEME_RECENT))
     {
         button_data->type = RECENT_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (nautilus_is_starred_directory (location))
+    else if (g_file_has_uri_scheme (location, SCHEME_STARRED))
     {
         button_data->type = STARRED_BUTTON;
         button_data->is_root = TRUE;
@@ -997,17 +1005,17 @@ setup_button_type (ButtonData      *button_data,
         button_data->type = MOUNT_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (nautilus_is_other_locations_directory (location))
+    else if (g_file_has_uri_scheme (location, SCHEME_OTHER_LOCATIONS))
     {
         button_data->type = OTHER_LOCATIONS_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (strcmp ((uri = g_file_get_uri (location)), "admin:///") == 0)
+    else if (nautilus_is_root_for_scheme (location, SCHEME_ADMIN))
     {
         button_data->type = ADMIN_ROOT_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (strcmp (uri, "trash:///") == 0)
+    else if (nautilus_is_root_for_scheme (location, SCHEME_TRASH))
     {
         button_data->type = TRASH_BUTTON;
         button_data->is_root = TRUE;
@@ -1028,7 +1036,6 @@ button_data_file_changed (NautilusFile *file,
     GFile *parent;
     GFile *button_parent;
     ButtonData *current_button_data;
-    char *display_name;
     NautilusPathBar *self;
     gboolean renamed;
     gboolean child;
@@ -1110,14 +1117,12 @@ button_data_file_changed (NautilusFile *file,
     /* MOUNTs use the GMount as the name, so don't update for those */
     if (button_data->type != MOUNT_BUTTON)
     {
-        display_name = nautilus_file_get_display_name (file);
+        const char *display_name = nautilus_file_get_display_name (file);
         if (g_strcmp0 (display_name, button_data->dir_name) != 0)
         {
             g_free (button_data->dir_name);
             button_data->dir_name = g_strdup (display_name);
         }
-
-        g_free (display_name);
     }
     current_dir = g_file_equal (self->current_path, button_data->path);
     nautilus_path_bar_update_button_appearance (button_data, current_dir);
@@ -1193,6 +1198,7 @@ make_button_data (NautilusPathBar *self,
         gtk_widget_add_css_class (button_data->button, "current-dir");
         gtk_widget_set_hexpand (button_data->button, TRUE);
         gtk_widget_set_halign (button_data->label, GTK_ALIGN_START);
+        gtk_actionable_set_action_name (GTK_ACTIONABLE (button_data->button), "win.enter-location");
     }
 
     if (button_data->label != NULL)
@@ -1219,7 +1225,7 @@ make_button_data (NautilusPathBar *self,
     }
     if (button_data->dir_name == NULL)
     {
-        button_data->dir_name = nautilus_file_get_display_name (file);
+        button_data->dir_name = g_strdup (nautilus_file_get_display_name (file));
     }
     if (button_data->file == NULL)
     {
@@ -1255,7 +1261,7 @@ make_button_data (NautilusPathBar *self,
 
     gtk_drop_target_set_preload (target, TRUE);
     /* TODO: Implement GDK_TYPE_STRING */
-    gtk_drop_target_set_gtypes (target, (GType[1]) { GDK_TYPE_FILE_LIST }, 1);
+    gtk_drop_target_set_gtypes (target, (GType[2]) { GDK_TYPE_TEXTURE, GDK_TYPE_FILE_LIST }, 2);
     g_signal_connect (target, "enter", G_CALLBACK (on_drag_motion), button_data);
     g_signal_connect (target, "motion", G_CALLBACK (on_drag_motion), button_data);
     g_signal_connect (target, "drop", G_CALLBACK (on_drag_drop), button_data);
