@@ -26,25 +26,22 @@
 #include "nautilus-preferences-window.h"
 
 #include <adwaita.h>
-#include <gtk/gtk.h>
-#include <gio/gio.h>
-
 #include <glib/gi18n.h>
-
 #include <nautilus-extension.h>
 
 #include "nautilus-column-utilities.h"
+#include "nautilus-date-utilities.h"
 #include "nautilus-global-preferences.h"
 
 /* bool preferences */
 #define NAUTILUS_PREFERENCES_DIALOG_FOLDERS_FIRST_WIDGET                       \
-        "sort_folders_first_switch"
+        "sort_folders_first_row"
 #define NAUTILUS_PREFERENCES_DIALOG_DELETE_PERMANENTLY_WIDGET                  \
-        "show_delete_permanently_switch"
+        "show_delete_permanently_row"
 #define NAUTILUS_PREFERENCES_DIALOG_CREATE_LINK_WIDGET                         \
-        "show_create_link_switch"
+        "show_create_link_row"
 #define NAUTILUS_PREFERENCES_DIALOG_LIST_VIEW_USE_TREE_WIDGET                  \
-        "use_tree_view_switch"
+        "use_tree_view_row"
 
 /* combo preferences */
 #define NAUTILUS_PREFERENCES_DIALOG_OPEN_ACTION_COMBO                          \
@@ -69,116 +66,103 @@ static const char * const icon_captions_components[] =
     "captions_0_comborow", "captions_1_comborow", "captions_2_comborow", NULL
 };
 
-static GtkWidget *preferences_window = NULL;
-
-static void list_store_append_string (GListStore  *list_store,
-                                      const gchar *string)
+static void
+create_icon_caption_combo_row_items (AdwComboRow *combo_row,
+                                     GList       *columns)
 {
-    g_autoptr (GtkStringObject) obj = gtk_string_object_new (string);
-    g_list_store_append (list_store, obj);
-}
-
-static void free_column_names_array(GPtrArray *column_names)
-{
-    g_ptr_array_foreach (column_names, (GFunc) g_free, NULL);
-    g_ptr_array_free (column_names, TRUE);
-}
-
-static void create_icon_caption_combo_row_items(AdwComboRow *combo_row,
-                                                GList       *columns)
-{
-    GListStore *list_store = g_list_store_new (GTK_TYPE_STRING_OBJECT);
+    g_autoptr (GListStore) list_store = g_list_store_new (NAUTILUS_TYPE_COLUMN);
+    g_autoptr (NautilusColumn) none = NULL;
+    g_autoptr (GtkExpression) expression = NULL;
     GList *l;
-    GPtrArray *column_names;
 
-    column_names = g_ptr_array_new ();
+    expression = gtk_property_expression_new (NAUTILUS_TYPE_COLUMN, NULL, "label");
+    adw_combo_row_set_expression (combo_row, expression);
 
-    /* Translators: this is referred to captions under icons. */
-    list_store_append_string (list_store, _("None"));
-    g_ptr_array_add (column_names, g_strdup ("none"));
+    none = g_object_new (NAUTILUS_TYPE_COLUMN,
+                         "name", "none",
+                         /* Translators: this is referred to captions under icons. */
+                         "label", _("None"),
+                         NULL);
+    g_list_store_append (list_store, none);
 
     for (l = columns; l != NULL; l = l->next)
     {
         NautilusColumn *column;
-        char *name;
-        char *label;
+        g_autofree char *name = NULL;
 
         column = NAUTILUS_COLUMN (l->data);
 
-        g_object_get (G_OBJECT (column), "name", &name, "label", &label, NULL);
+        g_object_get (G_OBJECT (column), "name", &name, NULL);
 
-        /* Don't show name here, it doesn't make sense */
-        if (!strcmp (name, "name"))
+        /* Don't show name here, it doesn't make sense
+         * starred is instead shown as an emblem for the grid view
+         */
+        if (!strcmp (name, "name") || !strcmp (name, "starred"))
         {
-            g_free (name);
-            g_free (label);
             continue;
         }
 
-        list_store_append_string (list_store, label);
-        g_ptr_array_add (column_names, name);
-
-        g_free (label);
+        g_list_store_append (list_store, column);
     }
     adw_combo_row_set_model (combo_row, G_LIST_MODEL (list_store));
-    g_object_set_data_full (G_OBJECT (combo_row), "column_names", column_names,
-                            (GDestroyNotify) free_column_names_array);
 }
 
-static void icon_captions_changed_callback(AdwComboRow *widget,
-                                           GParamSpec  *pspec,
-                                           gpointer     user_data)
+static void
+icon_captions_changed_callback (AdwComboRow *widget,
+                                GParamSpec  *pspec,
+                                gpointer     user_data)
 {
-    GPtrArray *captions;
-    GtkBuilder *builder;
-    int i;
+    g_autoptr (GStrvBuilder) builder = g_strv_builder_new ();
+    g_auto (GStrv) captions = NULL;
+    GPtrArray *combo_rows = (GPtrArray *) user_data;
 
-    builder = GTK_BUILDER (user_data);
-
-    captions = g_ptr_array_new ();
-
-    for (i = 0; icon_captions_components[i] != NULL; i++)
+    for (int i = 0; icon_captions_components[i] != NULL; i++)
     {
         GtkWidget *combo_row;
-        int selected_index;
-        GPtrArray *column_names;
-        char *name;
+        GObject *selected_column;
+        g_autofree char *name = NULL;
 
-        combo_row = GTK_WIDGET (
-            gtk_builder_get_object (builder, icon_captions_components[i]));
-        selected_index = adw_combo_row_get_selected (ADW_COMBO_ROW (combo_row));
+        combo_row = g_ptr_array_index (combo_rows, i);
+        selected_column = adw_combo_row_get_selected_item (ADW_COMBO_ROW (combo_row));
+        if (G_UNLIKELY (!NAUTILUS_IS_COLUMN (selected_column)))
+        {
+            g_warn_if_reached ();
+            continue;
+        }
 
-        column_names = g_object_get_data (G_OBJECT (combo_row), "column_names");
-
-        name = g_ptr_array_index (column_names, selected_index);
-        g_ptr_array_add (captions, name);
+        g_object_get (selected_column, "name", &name, NULL);
+        g_strv_builder_add (builder, name);
     }
-    g_ptr_array_add (captions, NULL);
+    captions = g_strv_builder_end (builder);
 
     g_settings_set_strv (nautilus_icon_view_preferences,
                          NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS,
-                         (const char **) captions->pdata);
-    g_ptr_array_free (captions, TRUE);
+                         (const char **) captions);
 }
 
-static void update_caption_combo_row(GtkBuilder *builder,
-                                     const char *combo_row_name,
-                                     const char *name)
+static void
+update_caption_combo_row (GPtrArray  *combo_rows,
+                          int         combo_row_i,
+                          const char *name)
 {
-    GtkWidget *combo_row;
-    int i;
-    GPtrArray *column_names;
+    AdwComboRow *combo_row;
+    GListModel *model;
+    guint n_columns;
 
-    combo_row = GTK_WIDGET (gtk_builder_get_object (builder, combo_row_name));
+    combo_row = ADW_COMBO_ROW (g_ptr_array_index (combo_rows, combo_row_i));
+    model = adw_combo_row_get_model (combo_row);
+    n_columns = g_list_model_get_n_items (model);
 
     g_signal_handlers_block_by_func (
-        combo_row, G_CALLBACK (icon_captions_changed_callback), builder);
+        combo_row, G_CALLBACK (icon_captions_changed_callback), combo_rows);
 
-    column_names = g_object_get_data (G_OBJECT (combo_row), "column_names");
-
-    for (i = 0; i < column_names->len; ++i)
+    for (guint i = 0; i < n_columns; ++i)
     {
-        if (!strcmp (name, g_ptr_array_index (column_names, i)))
+        g_autoptr (NautilusColumn) column_i = g_list_model_get_item (model, i);
+        g_autofree char *name_i = NULL;
+
+        g_object_get (column_i, "name", &name_i, NULL);
+        if (g_strcmp0 (name, name_i) == 0)
         {
             adw_combo_row_set_selected (ADW_COMBO_ROW (combo_row), i);
             break;
@@ -186,12 +170,13 @@ static void update_caption_combo_row(GtkBuilder *builder,
     }
 
     g_signal_handlers_unblock_by_func (
-        combo_row, G_CALLBACK (icon_captions_changed_callback), builder);
+        combo_row, G_CALLBACK (icon_captions_changed_callback), combo_rows);
 }
 
-static void update_icon_captions_from_settings(GtkBuilder *builder)
+static void
+update_icon_captions_from_settings (GPtrArray *combo_rows)
 {
-    char **captions;
+    g_auto (GStrv) captions = NULL;
     int i, j;
 
     captions = g_settings_get_strv (nautilus_icon_view_preferences,
@@ -203,7 +188,7 @@ static void update_icon_captions_from_settings(GtkBuilder *builder)
 
     for (i = 0, j = 0; icon_captions_components[i] != NULL; i++)
     {
-        char *data;
+        const char *data;
 
         if (captions[j])
         {
@@ -215,15 +200,14 @@ static void update_icon_captions_from_settings(GtkBuilder *builder)
             data = "none";
         }
 
-        update_caption_combo_row (builder, icon_captions_components[i], data);
+        update_caption_combo_row (combo_rows, i, data);
     }
-
-    g_strfreev (captions);
 }
 
 static void
 nautilus_preferences_window_setup_icon_caption_page (GtkBuilder *builder)
 {
+    g_autoptr (GPtrArray) combo_rows = g_ptr_array_sized_new (G_N_ELEMENTS (icon_captions_components));
     GList *columns;
     int i;
     gboolean writable;
@@ -239,41 +223,72 @@ nautilus_preferences_window_setup_icon_caption_page (GtkBuilder *builder)
 
         combo_row = GTK_WIDGET (
             gtk_builder_get_object (builder, icon_captions_components[i]));
+        g_ptr_array_add (combo_rows, combo_row);
 
         create_icon_caption_combo_row_items (ADW_COMBO_ROW (combo_row), columns);
         gtk_widget_set_sensitive (combo_row, writable);
 
         g_signal_connect_data (
             combo_row, "notify::selected", G_CALLBACK (icon_captions_changed_callback),
-            g_object_ref (builder), (GClosureNotify) g_object_unref, 0);
+            g_ptr_array_ref (combo_rows), (GClosureNotify) g_ptr_array_unref, 0);
     }
 
     nautilus_column_list_free (columns);
 
-    update_icon_captions_from_settings (builder);
+    update_icon_captions_from_settings (combo_rows);
 }
 
-static void bind_builder_bool(GtkBuilder *builder,
-                              GSettings  *settings,
-                              const char *widget_name,
-                              const char *prefs)
+static void
+bind_builder_bool (GtkBuilder *builder,
+                   GSettings  *settings,
+                   const char *widget_name,
+                   const char *prefs)
 {
     g_settings_bind (settings, prefs, gtk_builder_get_object (builder, widget_name),
                      "active", G_SETTINGS_BIND_DEFAULT);
 }
 
-static GVariant *combo_row_mapping_set(const GValue       *gvalue,
-                                       const GVariantType *expected_type,
-                                       gpointer            user_data)
+/* Translators: Both %s will be replaced with formatted timestamps. */
+#define DATE_FORMAT_ROW_SUBTITLE _("Examples: “%s”, “%s”")
+
+static void
+setup_detailed_date (GtkBuilder *builder)
+{
+    AdwActionRow *simple_row = ADW_ACTION_ROW (gtk_builder_get_object (builder, "date_format_simple_row"));
+    AdwActionRow *detailed_row = ADW_ACTION_ROW (gtk_builder_get_object (builder, "date_format_detailed_row"));
+
+    g_autoptr (GDateTime) now = g_date_time_new_now_local ();
+    g_autoptr (GDateTime) earlier = g_date_time_add_days (now, -3);
+
+    g_autofree gchar *simple_date_now = nautilus_date_preview_detailed_format (now, FALSE);
+    g_autofree gchar *simple_date_earlier = nautilus_date_preview_detailed_format (earlier, FALSE);
+    g_autofree gchar *simple_row_subtitle = g_strdup_printf (DATE_FORMAT_ROW_SUBTITLE,
+                                                             simple_date_now,
+                                                             simple_date_earlier);
+    adw_action_row_set_subtitle (simple_row, simple_row_subtitle);
+
+    g_autofree gchar *detailed_date_now = nautilus_date_preview_detailed_format (now, TRUE);
+    g_autofree gchar *detailed_date_earlier = nautilus_date_preview_detailed_format (earlier, TRUE);
+    g_autofree gchar *detailed_row_subtitle = g_strdup_printf (DATE_FORMAT_ROW_SUBTITLE,
+                                                               detailed_date_now,
+                                                               detailed_date_earlier);
+    adw_action_row_set_subtitle (detailed_row, detailed_row_subtitle);
+}
+
+static GVariant *
+combo_row_mapping_set (const GValue       *gvalue,
+                       const GVariantType *expected_type,
+                       gpointer            user_data)
 {
     const gchar **values = user_data;
 
     return g_variant_new_string (values[g_value_get_uint (gvalue)]);
 }
 
-static gboolean combo_row_mapping_get(GValue   *gvalue,
-                                      GVariant *variant,
-                                      gpointer  user_data)
+static gboolean
+combo_row_mapping_get (GValue   *gvalue,
+                       GVariant *variant,
+                       gpointer  user_data)
 {
     const gchar **values = user_data;
     const gchar *value;
@@ -293,11 +308,12 @@ static gboolean combo_row_mapping_get(GValue   *gvalue,
     return FALSE;
 }
 
-static void bind_builder_combo_row(GtkBuilder  *builder,
-                                   GSettings   *settings,
-                                   const char  *widget_name,
-                                   const char  *prefs,
-                                   const char **values)
+static void
+bind_builder_combo_row (GtkBuilder  *builder,
+                        GSettings   *settings,
+                        const char  *widget_name,
+                        const char  *prefs,
+                        const char **values)
 {
     g_settings_bind_with_mapping (settings, prefs, gtk_builder_get_object (builder, widget_name),
                                   "selected", G_SETTINGS_BIND_DEFAULT,
@@ -305,39 +321,32 @@ static void bind_builder_combo_row(GtkBuilder  *builder,
                                   (gpointer) values, NULL);
 }
 
-static void setup_combo (GtkBuilder  *builder,
-                         const char  *widget_name,
-                         const char **strings)
+static void
+setup_combo (GtkBuilder  *builder,
+             const char  *widget_name,
+             const char **strings)
 {
     AdwComboRow *combo_row;
-    GListStore *list_store;
+    g_autoptr (GtkStringList) list_store = NULL;
 
     combo_row = (AdwComboRow *) gtk_builder_get_object (builder, widget_name);
     g_assert (ADW_IS_COMBO_ROW (combo_row));
 
-    list_store = g_list_store_new (GTK_TYPE_STRING_OBJECT);
-
-    for (gsize i = 0; strings[i]; i++)
-    {
-        list_store_append_string (list_store, strings[i]);
-    }
-
+    list_store = gtk_string_list_new (strings);
     adw_combo_row_set_model (combo_row, G_LIST_MODEL (list_store));
 }
 
-static void nautilus_preferences_window_setup(GtkBuilder *builder,
-                                              GtkWindow  *parent_window)
+static void
+nautilus_preferences_window_setup (GtkBuilder *builder)
 {
-    GtkWidget *window;
-
     setup_combo (builder, NAUTILUS_PREFERENCES_DIALOG_OPEN_ACTION_COMBO,
-                 (const char *[]) { _("Single click"), _("Double click"), NULL });
+                 (const char *[]) { _("Single-Click"), _("Double-Click"), NULL });
     setup_combo (builder, NAUTILUS_PREFERENCES_DIALOG_SEARCH_RECURSIVE_ROW,
-                 (const char *[]) { _("On this computer only"), _("All locations"), _("Never"), NULL });
+                 (const char *[]) { _("On This Computer Only"), _("All Locations"), _("Never"), NULL });
     setup_combo (builder, NAUTILUS_PREFERENCES_DIALOG_THUMBNAILS_ROW,
-                 (const char *[]) { _("On this computer only"), _("All files"), _("Never"), NULL });
+                 (const char *[]) { _("On This Computer Only"), _("All Files"), _("Never"), NULL });
     setup_combo (builder, NAUTILUS_PREFERENCES_DIALOG_COUNT_ROW,
-                 (const char *[]) { _("On this computer only"), _("All folders"), _("Never"), NULL });
+                 (const char *[]) { _("On This Computer Only"), _("All Folders"), _("Never"), NULL });
 
     /* setup preferences */
     bind_builder_bool (builder, gtk_filechooser_preferences,
@@ -353,6 +362,8 @@ static void nautilus_preferences_window_setup(GtkBuilder *builder,
                        NAUTILUS_PREFERENCES_DIALOG_DELETE_PERMANENTLY_WIDGET,
                        NAUTILUS_PREFERENCES_SHOW_DELETE_PERMANENTLY);
 
+    setup_detailed_date (builder);
+
     bind_builder_combo_row (builder, nautilus_preferences,
                             NAUTILUS_PREFERENCES_DIALOG_OPEN_ACTION_COMBO,
                             NAUTILUS_PREFERENCES_CLICK_POLICY,
@@ -371,42 +382,37 @@ static void nautilus_preferences_window_setup(GtkBuilder *builder,
                             (const char **) speed_tradeoff_values);
 
     nautilus_preferences_window_setup_icon_caption_page (builder);
-
-    /* UI callbacks */
-    window = GTK_WIDGET (gtk_builder_get_object (builder, "preferences_window"));
-    preferences_window = window;
-
-    gtk_window_set_icon_name (GTK_WINDOW (preferences_window), APPLICATION_ID);
-
-    g_object_add_weak_pointer (G_OBJECT (window), (gpointer *) &preferences_window);
-
-    gtk_window_set_transient_for (GTK_WINDOW (preferences_window), parent_window);
-
-    gtk_window_present (GTK_WINDOW (preferences_window));
 }
 
-void nautilus_preferences_window_show(GtkWindow *window)
+void
+nautilus_preferences_window_show (GtkWindow *parent_window)
 {
-    GtkBuilder *builder;
-    g_autoptr (GError) error = NULL;
+    static GtkWindow *preferences_window = NULL;
+    g_autoptr (GtkBuilder) builder = NULL;
+    g_autoptr (GSimpleActionGroup) action_group = g_simple_action_group_new ();
+    g_autoptr (GAction) date_time_action = NULL;
 
     if (preferences_window != NULL)
     {
         /* Destroy existing window, which might be hidden behind other windows,
          * attached to another parent. */
-        gtk_window_destroy (GTK_WINDOW (preferences_window));
+        gtk_window_destroy (preferences_window);
     }
 
-    builder = gtk_builder_new ();
+    builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/ui/nautilus-preferences-window.ui");
+    nautilus_preferences_window_setup (builder);
 
-    gtk_builder_add_from_resource (
-        builder, "/org/gnome/nautilus/ui/nautilus-preferences-window.ui", &error);
-    if (error != NULL)
-    {
-        g_error ("Failed to add nautilus-preferences-window.ui: %s", error->message);
-    }
+    preferences_window = GTK_WINDOW (gtk_builder_get_object (builder, "preferences_window"));
+    g_object_add_weak_pointer (G_OBJECT (preferences_window), (gpointer *) &preferences_window);
 
-    nautilus_preferences_window_setup (builder, window);
+    date_time_action = g_settings_create_action (nautilus_preferences,
+                                                 NAUTILUS_PREFERENCES_DATE_TIME_FORMAT);
+    g_action_map_add_action (G_ACTION_MAP (action_group), date_time_action);
+    gtk_widget_insert_action_group (GTK_WIDGET (preferences_window),
+                                    "preferences",
+                                    G_ACTION_GROUP (action_group));
 
-    g_object_unref (builder);
+    gtk_window_set_icon_name (preferences_window, APPLICATION_ID);
+    gtk_window_set_transient_for (preferences_window, parent_window);
+    gtk_window_present (preferences_window);
 }
