@@ -9,11 +9,23 @@
 #include "nautilus-file-operations.h"
 #include "nautilus-progress-info-manager.h"
 #include "nautilus-progress-info-widget.h"
+#include "nautilus-progress-paintable.h"
 #include "nautilus-window.h"
 
 #define OPERATION_MINIMUM_TIME 2 /*s */
 #define NEEDS_ATTENTION_ANIMATION_TIMEOUT 2000 /*ms */
 #define REMOVE_FINISHED_OPERATIONS_TIEMOUT 3 /*s */
+#define MAX_ITEMS_IN_SIDEBAR 4
+
+enum
+{
+    PROP_0,
+    PROP_POPOVER_POSITION,
+    PROP_REVEAL,
+    N_PROPS
+};
+
+static GParamSpec *properties[N_PROPS] = { NULL, };
 
 struct _NautilusProgressIndicator
 {
@@ -27,8 +39,10 @@ struct _NautilusProgressIndicator
     GtkWidget *operations_popover;
     GtkWidget *operations_list;
     GListStore *progress_infos_model;
-    GtkWidget *operations_revealer;
-    GtkWidget *operations_icon;
+    GtkWidget *sidebar_list;
+
+    gboolean reveal;
+    GtkPositionType popover_position;
 
     NautilusProgressInfoManager *progress_manager;
 };
@@ -38,7 +52,7 @@ G_DEFINE_FINAL_TYPE (NautilusProgressIndicator, nautilus_progress_indicator, ADW
 
 static void update_operations (NautilusProgressIndicator *self);
 
-static gboolean
+static inline gboolean
 should_show_progress_info (NautilusProgressInfo *info)
 {
     return nautilus_progress_info_get_total_elapsed_time (info) +
@@ -69,35 +83,38 @@ get_filtered_progress_infos (NautilusProgressIndicator *self)
 static gboolean
 should_hide_operations_button (NautilusProgressIndicator *self)
 {
-    GList *progress_infos;
-    GList *l;
+    g_autoptr (GList) progress_infos = get_filtered_progress_infos (self);
 
-    progress_infos = get_filtered_progress_infos (self);
-
-    for (l = progress_infos; l != NULL; l = l->next)
+    for (GList *l = progress_infos; l != NULL; l = l->next)
     {
-        if (nautilus_progress_info_get_total_elapsed_time (l->data) +
-            nautilus_progress_info_get_remaining_time (l->data) > OPERATION_MINIMUM_TIME &&
-            !nautilus_progress_info_get_is_cancelled (l->data) &&
+        if (!nautilus_progress_info_get_is_cancelled (l->data) &&
             !nautilus_progress_info_get_is_finished (l->data))
         {
             return FALSE;
         }
     }
 
-    g_list_free (progress_infos);
-
     return TRUE;
 }
 
+static void
+nautilus_progress_indicator_set_reveal (NautilusProgressIndicator *self,
+                                        gboolean                   reveal)
+{
+    if (reveal != self->reveal)
+    {
+        self->reveal = reveal;
+        g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_REVEAL]);
+    }
+}
+
 static gboolean
 on_remove_finished_operations_timeout (NautilusProgressIndicator *self)
 {
     nautilus_progress_info_manager_remove_finished_or_cancelled_infos (self->progress_manager);
     if (should_hide_operations_button (self))
     {
-        gtk_revealer_set_reveal_child (GTK_REVEALER (self->operations_revealer),
-                                       FALSE);
+        nautilus_progress_indicator_set_reveal (self, FALSE);
     }
     else
     {
@@ -134,8 +151,7 @@ schedule_remove_finished_operations (NautilusProgressIndicator *self)
 static void
 remove_operations_button_attention_style (NautilusProgressIndicator *self)
 {
-    gtk_widget_remove_css_class (self->operations_button,
-                                 "nautilus-operations-button-needs-attention");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "needs-attention");
 }
 
 static gboolean
@@ -163,8 +179,7 @@ add_operations_button_attention_style (NautilusProgressIndicator *self)
     unschedule_operations_button_attention_style (self);
     remove_operations_button_attention_style (self);
 
-    gtk_widget_add_css_class (self->operations_button,
-                              "nautilus-operations-button-needs-attention");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "needs-attention");
     self->operations_button_attention_timeout_id = g_timeout_add (NEEDS_ATTENTION_ANIMATION_TIMEOUT,
                                                                   (GSourceFunc) on_remove_operations_button_attention_style_timeout,
                                                                   self);
@@ -173,22 +188,12 @@ add_operations_button_attention_style (NautilusProgressIndicator *self)
 static void
 on_progress_info_cancelled (NautilusProgressIndicator *self)
 {
-    /* Update the pie chart progress */
-    gtk_widget_queue_draw (self->operations_icon);
-
     if (!nautilus_progress_manager_has_viewers (self->progress_manager))
     {
         schedule_remove_finished_operations (self);
     }
 }
 
-static void
-on_progress_info_progress_changed (NautilusProgressIndicator *self)
-{
-    /* Update the pie chart progress */
-    gtk_widget_queue_draw (self->operations_icon);
-}
-
 static void
 on_progress_info_finished (NautilusProgressIndicator *self,
                            NautilusProgressInfo      *info)
@@ -199,9 +204,6 @@ on_progress_info_finished (NautilusProgressIndicator *self,
 
     window = NAUTILUS_WINDOW (gtk_widget_get_root (GTK_WIDGET (self)));
 
-    /* Update the pie chart progress */
-    gtk_widget_queue_draw (self->operations_icon);
-
     if (!nautilus_progress_manager_has_viewers (self->progress_manager))
     {
         schedule_remove_finished_operations (self);
@@ -214,64 +216,63 @@ on_progress_info_finished (NautilusProgressIndicator *self,
     if (!gtk_widget_is_visible (self->operations_popover) &&
         folder_to_open != NULL)
     {
-        add_operations_button_attention_style (self);
+        gboolean was_quick = nautilus_progress_info_get_total_elapsed_time (info) <= OPERATION_MINIMUM_TIME;
+        if (!was_quick)
+        {
+            add_operations_button_attention_style (self);
+        }
+
         main_label = nautilus_progress_info_get_status (info);
         nautilus_window_show_operation_notification (window,
                                                      main_label,
-                                                     folder_to_open);
+                                                     folder_to_open,
+                                                     was_quick);
         g_free (main_label);
     }
 
     g_clear_object (&folder_to_open);
 }
 
-static void
-disconnect_progress_infos (NautilusProgressIndicator *self)
+static GdkPaintable *
+get_paintable (GtkListItem          *listitem,
+               NautilusProgressInfo *info)
 {
-    GList *progress_infos;
-    GList *l;
-
-    progress_infos = nautilus_progress_info_manager_get_all_infos (self->progress_manager);
-    for (l = progress_infos; l != NULL; l = l->next)
+    if (info == NULL)
     {
-        g_signal_handlers_disconnect_by_data (l->data, self);
+        return NULL;
     }
+
+    GtkWidget *box = gtk_list_item_get_child (listitem);
+    GtkWidget *image = gtk_widget_get_first_child (box);
+
+    GdkPaintable *paintable = nautilus_progress_paintable_new (image);
+
+    g_object_bind_property (info, "icon-name", paintable, "icon-name", G_BINDING_SYNC_CREATE);
+    g_signal_connect_object (info, "finished", G_CALLBACK (nautilus_progress_paintable_animate_done),
+                             paintable, G_CONNECT_SWAPPED);
+
+    g_object_bind_property (info, "progress", paintable, "progress", G_BINDING_SYNC_CREATE);
+
+    return paintable;
 }
 
 static void
 update_operations (NautilusProgressIndicator *self)
 {
-    GList *progress_infos;
-    GList *l;
-    gboolean should_show_progress_button = FALSE;
+    g_autoptr (GList) progress_infos = get_filtered_progress_infos (self);
+    gboolean should_show_progress_button = (progress_infos != NULL);
 
-    disconnect_progress_infos (self);
     g_list_store_remove_all (self->progress_infos_model);
 
-    progress_infos = get_filtered_progress_infos (self);
-    for (l = progress_infos; l != NULL; l = l->next)
+    for (GList *l = progress_infos; l != NULL; l = l->next)
     {
-        should_show_progress_button = should_show_progress_button ||
-                                      should_show_progress_info (l->data);
-
-        g_signal_connect_object (l->data, "finished",
-                                 G_CALLBACK (on_progress_info_finished), self, G_CONNECT_SWAPPED);
-        g_signal_connect_object (l->data, "cancelled",
-                                 G_CALLBACK (on_progress_info_cancelled), self, G_CONNECT_SWAPPED);
-        g_signal_connect_object (l->data, "progress-changed",
-                                 G_CALLBACK (on_progress_info_progress_changed), self, G_CONNECT_SWAPPED);
         g_list_store_append (self->progress_infos_model, l->data);
     }
 
-    g_list_free (progress_infos);
-
-    if (should_show_progress_button &&
-        !gtk_revealer_get_reveal_child (GTK_REVEALER (self->operations_revealer)))
+    if (should_show_progress_button && !self->reveal)
     {
         add_operations_button_attention_style (self);
-        gtk_revealer_set_reveal_child (GTK_REVEALER (self->operations_revealer),
-                                       TRUE);
-        gtk_widget_queue_draw (self->operations_icon);
+        nautilus_progress_indicator_set_reveal (self, TRUE);
     }
 
     /* Since we removed the info widgets, we need to restore the focus */
@@ -334,10 +335,8 @@ unschedule_operations_start (NautilusProgressIndicator *self)
 }
 
 static void
-on_progress_info_started (NautilusProgressInfo      *info,
-                          NautilusProgressIndicator *self)
+on_progress_info_started (NautilusProgressIndicator *self)
 {
-    g_signal_handlers_disconnect_by_data (info, self);
     schedule_operations_start (self);
 }
 
@@ -346,82 +345,15 @@ on_new_progress_info (NautilusProgressInfoManager *manager,
                       NautilusProgressInfo        *info,
                       NautilusProgressIndicator   *self)
 {
-    g_signal_connect (info, "started",
-                      G_CALLBACK (on_progress_info_started), self);
-}
-
-static void
-on_operations_icon_draw (GtkDrawingArea            *drawing_area,
-                         cairo_t                   *cr,
-                         int                        width,
-                         int                        height,
-                         NautilusProgressIndicator *self)
-{
-    GtkWidget *widget = GTK_WIDGET (drawing_area);
-    gfloat elapsed_progress = 0;
-    gint remaining_progress = 0;
-    gint total_progress;
-    gdouble ratio;
-    GList *progress_infos;
-    GList *l;
-    gboolean all_cancelled;
-    GdkRGBA background;
-    GdkRGBA foreground;
-
-    gtk_widget_get_color (widget, &foreground);
-    background = foreground;
-    background.alpha *= 0.3;
-
-    all_cancelled = TRUE;
-    progress_infos = get_filtered_progress_infos (self);
-    for (l = progress_infos; l != NULL; l = l->next)
-    {
-        if (!nautilus_progress_info_get_is_cancelled (l->data))
-        {
-            all_cancelled = FALSE;
-            remaining_progress += nautilus_progress_info_get_remaining_time (l->data);
-            elapsed_progress += nautilus_progress_info_get_elapsed_time (l->data);
-        }
-    }
-
-    g_list_free (progress_infos);
-
-    total_progress = remaining_progress + elapsed_progress;
-
-    if (all_cancelled)
-    {
-        ratio = 1.0;
-    }
-    else
-    {
-        if (total_progress > 0)
-        {
-            ratio = MAX (0.05, elapsed_progress / total_progress);
-        }
-        else
-        {
-            ratio = 0.05;
-        }
-    }
-
-
-    width = gtk_widget_get_allocated_width (widget);
-    height = gtk_widget_get_allocated_height (widget);
-
-    gdk_cairo_set_source_rgba (cr, &background);
-    cairo_arc (cr,
-               width / 2.0, height / 2.0,
-               MIN (width, height) / 2.0,
-               0, 2 * G_PI);
-    cairo_fill (cr);
-    cairo_move_to (cr, width / 2.0, height / 2.0);
-    gdk_cairo_set_source_rgba (cr, &foreground);
-    cairo_arc (cr,
-               width / 2.0, height / 2.0,
-               MIN (width, height) / 2.0,
-               -G_PI / 2.0, ratio * 2 * G_PI - G_PI / 2.0);
-
-    cairo_fill (cr);
+    g_signal_connect_object (info, "started",
+                             G_CALLBACK (on_progress_info_started), self,
+                             G_CONNECT_SWAPPED);
+    g_signal_connect_object (info, "finished",
+                             G_CALLBACK (on_progress_info_finished), self,
+                             G_CONNECT_SWAPPED);
+    g_signal_connect_object (info, "cancelled",
+                             G_CALLBACK (on_progress_info_cancelled), self,
+                             G_CONNECT_SWAPPED);
 }
 
 static void
@@ -472,16 +404,89 @@ operations_list_create_widget (GObject  *item,
     return widget;
 }
 
+static void
+direction_changed (GtkWidget        *self,
+                   GtkTextDirection  previous_direction,
+                   gpointer          user_data)
+{
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_POPOVER_POSITION]);
+}
+
+static void
+nautilus_progress_indicator_get_property (GObject    *object,
+                                          guint       property_id,
+                                          GValue     *value,
+                                          GParamSpec *pspec)
+{
+    NautilusProgressIndicator *self = NAUTILUS_PROGRESS_INDICATOR (object);
+
+    switch (property_id)
+    {
+        case (PROP_POPOVER_POSITION):
+        {
+            /* Handles the default value of zero coming from the breakpoint */
+            if (self->popover_position <= GTK_POS_RIGHT)
+            {
+                guint popover_position = (gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL) ?
+                                         GTK_POS_RIGHT : GTK_POS_LEFT;
+
+                g_value_set_enum (value, popover_position);
+            }
+            else
+            {
+                g_value_set_enum (value, self->popover_position);
+            }
+        }
+        break;
+
+        case (PROP_REVEAL):
+        {
+            g_value_set_boolean (value, self->reveal);
+        }
+        break;
+
+        default:
+        {
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        }
+    }
+}
+
+static void
+nautilus_progress_indicator_set_property (GObject      *object,
+                                          guint         property_id,
+                                          const GValue *value,
+                                          GParamSpec   *pspec)
+{
+    NautilusProgressIndicator *self = NAUTILUS_PROGRESS_INDICATOR (object);
+
+    switch (property_id)
+    {
+        case (PROP_POPOVER_POSITION):
+        {
+            self->popover_position = g_value_get_enum (value);
+        }
+        break;
+
+        default:
+        {
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        }
+    }
+}
+
 static void
 nautilus_progress_indicator_constructed (GObject *object)
 {
     NautilusProgressIndicator *self = NAUTILUS_PROGRESS_INDICATOR (object);
 
     self->progress_manager = nautilus_progress_info_manager_dup_singleton ();
-    g_signal_connect (self->progress_manager, "new-progress-info",
-                      G_CALLBACK (on_new_progress_info), self);
-    g_signal_connect (self->progress_manager, "has-viewers-changed",
-                      G_CALLBACK (on_progress_has_viewers_changed), self);
+    g_signal_connect_object (self->progress_manager, "new-progress-info",
+                             G_CALLBACK (on_new_progress_info), self,
+                             G_CONNECT_DEFAULT);
+    g_signal_connect_object (self->progress_manager, "has-viewers-changed",
+                             G_CALLBACK (on_progress_has_viewers_changed), self,
+                             G_CONNECT_DEFAULT);
 
     self->progress_infos_model = g_list_store_new (NAUTILUS_TYPE_PROGRESS_INFO);
     gtk_list_box_bind_model (GTK_LIST_BOX (self->operations_list),
@@ -489,26 +494,42 @@ nautilus_progress_indicator_constructed (GObject *object)
                              (GtkListBoxCreateWidgetFunc) operations_list_create_widget,
                              NULL,
                              NULL);
+
     update_operations (self);
 
+    GtkSliceListModel *slice = gtk_slice_list_model_new (g_object_ref (G_LIST_MODEL (self->progress_infos_model)),
+                                                         0, MAX_ITEMS_IN_SIDEBAR);
+    g_autoptr (GtkNoSelection) selection_model = gtk_no_selection_new (G_LIST_MODEL (slice));
+
+    gtk_list_view_set_model (GTK_LIST_VIEW (self->sidebar_list), GTK_SELECTION_MODEL (selection_model));
+
     g_signal_connect (self->operations_popover, "show",
                       (GCallback) gtk_widget_grab_focus, NULL);
     g_signal_connect_swapped (self->operations_popover, "closed",
                               (GCallback) gtk_widget_grab_focus, self);
 }
 
+static void
+nautilus_progress_indicator_dispose (GObject *obj)
+{
+    NautilusProgressIndicator *self = NAUTILUS_PROGRESS_INDICATOR (obj);
+
+    adw_bin_set_child (ADW_BIN (self), NULL);
+    gtk_widget_dispose_template (GTK_WIDGET (self), NAUTILUS_TYPE_PROGRESS_INDICATOR);
+
+    G_OBJECT_CLASS (nautilus_progress_indicator_parent_class)->dispose (obj);
+}
+
 static void
 nautilus_progress_indicator_finalize (GObject *obj)
 {
     NautilusProgressIndicator *self = NAUTILUS_PROGRESS_INDICATOR (obj);
 
-    disconnect_progress_infos (self);
     unschedule_remove_finished_operations (self);
     unschedule_operations_start (self);
     unschedule_operations_button_attention_style (self);
 
     g_clear_object (&self->progress_infos_model);
-    g_signal_handlers_disconnect_by_data (self->progress_manager, self);
     g_clear_object (&self->progress_manager);
 
     G_OBJECT_CLASS (nautilus_progress_indicator_parent_class)->finalize (obj);
@@ -521,17 +542,28 @@ nautilus_progress_indicator_class_init (NautilusProgressIndicatorClass *klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
     object_class->constructed = nautilus_progress_indicator_constructed;
+    object_class->dispose = nautilus_progress_indicator_dispose;
     object_class->finalize = nautilus_progress_indicator_finalize;
+    object_class->get_property = nautilus_progress_indicator_get_property;
+    object_class->set_property = nautilus_progress_indicator_set_property;
 
     gtk_widget_class_set_template_from_resource (widget_class,
                                                  "/org/gnome/nautilus/ui/nautilus-progress-indicator.ui");
     gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, operations_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, operations_icon);
     gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, operations_popover);
     gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, operations_list);
-    gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, operations_revealer);
+    gtk_widget_class_bind_template_child (widget_class, NautilusProgressIndicator, sidebar_list);
 
+    gtk_widget_class_bind_template_callback (widget_class, get_paintable);
     gtk_widget_class_bind_template_callback (widget_class, on_operations_popover_notify_visible);
+
+    properties[PROP_POPOVER_POSITION] = g_param_spec_enum ("popover-position", NULL, NULL,
+                                                           GTK_TYPE_POSITION_TYPE, 0,
+                                                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_REVEAL] = g_param_spec_boolean ("reveal", NULL, NULL, FALSE,
+                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+    g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
@@ -539,8 +571,5 @@ nautilus_progress_indicator_init (NautilusProgressIndicator *self)
 {
     gtk_widget_init_template (GTK_WIDGET (self));
 
-    gtk_drawing_area_set_draw_func (GTK_DRAWING_AREA (self->operations_icon),
-                                    (GtkDrawingAreaDrawFunc) on_operations_icon_draw,
-                                    self,
-                                    NULL);
+    g_signal_connect (self, "direction-changed", G_CALLBACK (direction_changed), NULL);
 }
