@@ -41,6 +41,7 @@ struct _NautilusQueryEditor
 {
     GtkWidget parent_instance;
 
+    GtkWidget *prefix_icon;
     GtkWidget *tags_box;
     GtkWidget *text;
     GtkWidget *clear_icon;
@@ -328,19 +329,12 @@ static void
 create_query (NautilusQueryEditor *editor)
 {
     NautilusQuery *query;
-    g_autoptr (NautilusFile) file = NULL;
     gboolean fts_enabled;
 
     g_return_if_fail (editor->query == NULL);
 
     fts_enabled = nautilus_search_popover_get_fts_enabled (NAUTILUS_SEARCH_POPOVER (editor->popover));
 
-    if (editor->location == NULL)
-    {
-        return;
-    }
-
-    file = nautilus_file_get (editor->location);
     query = nautilus_query_new ();
 
     nautilus_query_set_search_content (query, fts_enabled);
@@ -367,12 +361,9 @@ static gboolean
 entry_changed_internal (NautilusQueryEditor *editor)
 {
     const gchar *text = gtk_editable_get_text (GTK_EDITABLE (editor->text));
-    gboolean is_empty = (text == NULL && *text == '\0');
 
     editor->search_changed_idle_id = 0;
 
-    gtk_widget_set_child_visible (editor->clear_icon, !is_empty);
-
     if (editor->query == NULL)
     {
         create_query (editor);
@@ -392,6 +383,11 @@ static void
 entry_changed_cb (GtkWidget           *entry,
                   NautilusQueryEditor *editor)
 {
+    const gchar *text = gtk_editable_get_text (GTK_EDITABLE (editor->text));
+
+    gboolean is_empty = (text == NULL || *text == '\0');
+    gtk_widget_set_child_visible (editor->clear_icon, !is_empty);
+
     g_clear_handle_id (&editor->search_changed_idle_id, g_source_remove);
 
     if (editor->change_frozen)
@@ -588,7 +584,6 @@ static void
 nautilus_query_editor_init (NautilusQueryEditor *editor)
 {
     gboolean rtl = (gtk_widget_get_direction (GTK_WIDGET (editor)) == GTK_TEXT_DIR_RTL);
-    GtkWidget *image;
     GtkEventController *controller;
 
     gtk_widget_set_name (GTK_WIDGET (editor), "NautilusQueryEditor");
@@ -600,25 +595,24 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
                       editor);
 
     /* create the search entry */
-    image = gtk_image_new_from_icon_name ("system-search-symbolic");
-    g_object_set (image, "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION, NULL);
-    gtk_widget_set_margin_start (image, 4);
-    gtk_widget_set_margin_end (image, 6);
-    gtk_widget_set_parent (image, GTK_WIDGET (editor));
+    editor->prefix_icon = gtk_image_new ();
+    g_object_set (editor->prefix_icon, "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION, NULL);
+    gtk_widget_set_margin_start (editor->prefix_icon, 4);
+    gtk_widget_set_margin_end (editor->prefix_icon, 6);
+    gtk_widget_set_parent (editor->prefix_icon, GTK_WIDGET (editor));
 
     editor->tags_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_widget_set_visible (editor->tags_box, FALSE);
     gtk_widget_set_parent (editor->tags_box, GTK_WIDGET (editor));
 
     editor->text = gtk_text_new ();
-    gtk_text_set_placeholder_text (GTK_TEXT (editor->text), _("Search files and folders"));
     gtk_widget_set_hexpand (editor->text, TRUE);
     gtk_widget_set_parent (editor->text, GTK_WIDGET (editor));
 
     editor->clear_icon = gtk_image_new_from_icon_name (rtl ? "edit-clear-rtl-symbolic" :
                                                              "edit-clear-symbolic");
     g_object_set (editor->clear_icon, "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION, NULL);
-    gtk_widget_set_tooltip_text (editor->clear_icon, _("Clear entry"));
+    gtk_widget_set_tooltip_text (editor->clear_icon, _("Clear Entry"));
     gtk_widget_set_child_visible (editor->clear_icon, FALSE);
     gtk_widget_set_parent (editor->clear_icon, GTK_WIDGET (editor));
 
@@ -641,7 +635,7 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
 
     /* setup the filter menu button */
     editor->dropdown_button = gtk_menu_button_new ();
-    gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (editor->dropdown_button), "funnel-symbolic");
+    gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (editor->dropdown_button), "nautilus-search-filters-symbolic");
     gtk_widget_set_tooltip_text (GTK_WIDGET (editor->dropdown_button), _("Filter Search Results"));
     gtk_menu_button_set_popover (GTK_MENU_BUTTON (editor->dropdown_button), editor->popover);
     gtk_widget_set_parent (editor->dropdown_button, GTK_WIDGET (editor));
@@ -672,19 +666,6 @@ nautilus_query_editor_changed (NautilusQueryEditor *editor)
     g_signal_emit (editor, signals[CHANGED], 0, editor->query, TRUE);
 }
 
-NautilusQuery *
-nautilus_query_editor_get_query (NautilusQueryEditor *editor)
-{
-    g_return_val_if_fail (NAUTILUS_IS_QUERY_EDITOR (editor), NULL);
-
-    if (editor->text == NULL)
-    {
-        return NULL;
-    }
-
-    return editor->query;
-}
-
 GtkWidget *
 nautilus_query_editor_new (void)
 {
@@ -727,6 +708,15 @@ nautilus_query_editor_set_location (NautilusQueryEditor *editor,
 
     update_fts_sensitivity (editor);
 
+    gtk_image_set_from_icon_name (GTK_IMAGE (editor->prefix_icon),
+                                  (editor->location != NULL ?
+                                   "nautilus-folder-search-symbolic" :
+                                   "edit-find-symbolic"));
+    gtk_text_set_placeholder_text (GTK_TEXT (editor->text),
+                                   (editor->location != NULL ?
+                                    _("Search current folder") :
+                                    _("Search everywhere")));
+
     if (should_notify)
     {
         g_object_notify (G_OBJECT (editor), "location");
