@@ -6,6 +6,8 @@
 
 #pragma once
 
+#include <glib.h>
+
 #define SCHEME_ADMIN "admin"
 #define SCHEME_BURN "burn"
 #define SCHEME_SEARCH "x-nautilus-search"
@@ -14,3 +16,6 @@
 #define SCHEME_RECENT "recent"
 #define SCHEME_STARRED "starred"
 #define SCHEME_TRASH "trash"
+
+gboolean
+nautilus_scheme_is_internal (const char *scheme);
