@@ -29,6 +29,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <gio/gunixmounts.h>
 #include <gio/gio.h>
 
 #define BATCH_SIZE 500
@@ -483,12 +484,20 @@ static gpointer
 search_thread_func (gpointer user_data)
 {
     SearchThreadData *data;
+    g_autoptr (GUnixMountMonitor) mount_monitor = NULL;
     GFile *dir;
     GFileInfo *info;
     const char *id;
 
     data = user_data;
 
+    if (nautilus_query_get_recursive (data->query) == NAUTILUS_QUERY_RECURSIVE_LOCAL_ONLY)
+    {
+        /* Need to retain a mount monitor so that filesystem queries used to
+         * determine if we recurse are done fast. */
+        mount_monitor = g_unix_mount_monitor_get ();
+    }
+
     /* Insert id for toplevel directory into visited */
     dir = g_queue_peek_head (data->directories);
     info = g_file_query_info (dir, G_FILE_ATTRIBUTE_ID_FILE, 0, data->cancellable, NULL);
