@@ -18,6 +18,7 @@
  * Author: Jamie McCracken <jamiemcc@gnome.org>
  *
  */
+#define G_LOG_DOMAIN "nautilus-search"
 
 #include <config.h>
 #include "nautilus-search-engine-tracker.h"
@@ -26,8 +27,6 @@
 #include "nautilus-search-hit.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-tracker-utilities.h"
-#define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH
-#include "nautilus-debug.h"
 
 #include <string.h>
 #include <gio/gio.h>
@@ -109,7 +108,7 @@ check_pending_hits (NautilusSearchEngineTracker *tracker,
     GList *hits = NULL;
     NautilusSearchHit *hit;
 
-    DEBUG ("Tracker engine add hits");
+    g_debug ("Tracker engine add hits");
 
     if (!force_send &&
         g_queue_get_length (tracker->hits_pending) < BATCH_SIZE)
@@ -130,7 +129,7 @@ static void
 search_finished (NautilusSearchEngineTracker *tracker,
                  GError                      *error)
 {
-    DEBUG ("Tracker engine finished");
+    g_debug ("Tracker engine finished");
 
     if (error == NULL)
     {
@@ -148,7 +147,7 @@ search_finished (NautilusSearchEngineTracker *tracker,
 
     if (error && !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
     {
-        DEBUG ("Tracker engine error %s", error->message);
+        g_debug ("Tracker engine error %s", error->message);
         nautilus_search_provider_error (NAUTILUS_SEARCH_PROVIDER (tracker), error->message);
     }
     else
@@ -157,11 +156,11 @@ search_finished (NautilusSearchEngineTracker *tracker,
                                            NAUTILUS_SEARCH_PROVIDER_STATUS_NORMAL);
         if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         {
-            DEBUG ("Tracker engine finished and cancelled");
+            g_debug ("Tracker engine finished and cancelled");
         }
         else
         {
-            DEBUG ("Tracker engine finished correctly");
+            g_debug ("Tracker engine finished correctly");
         }
     }
 
@@ -320,7 +319,7 @@ search_finished_idle (gpointer user_data)
 {
     NautilusSearchEngineTracker *tracker = user_data;
 
-    DEBUG ("Tracker engine finished idle");
+    g_debug ("Tracker engine finished idle");
 
     search_finished (tracker, NULL);
 
@@ -491,7 +490,7 @@ nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
         return;
     }
 
-    DEBUG ("Tracker engine start");
+    g_debug ("Tracker engine start");
     g_object_ref (tracker);
     tracker->query_pending = TRUE;
 
@@ -572,9 +571,8 @@ nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
     if (mimetypes->len > 0)
     {
         g_autoptr (GString) mimetype_str = NULL;
-        gint i;
 
-        for (i = 0; i < mimetypes->len; i++)
+        for (guint i = 0; i < mimetypes->len; i++)
         {
             const gchar *mimetype;
 
@@ -632,7 +630,7 @@ nautilus_search_engine_tracker_stop (NautilusSearchProvider *provider)
 
     if (tracker->query_pending)
     {
-        DEBUG ("Tracker engine stop");
+        g_debug ("Tracker engine stop");
         g_cancellable_cancel (tracker->cancellable);
         g_clear_object (&tracker->cancellable);
         tracker->query_pending = FALSE;
