@@ -16,6 +16,7 @@
  * see <http://www.gnu.org/licenses/>.
  *
  */
+#define G_LOG_DOMAIN "nautilus-search-hit"
 
 #include <config.h>
 
@@ -24,8 +25,6 @@
 
 #include "nautilus-search-hit.h"
 #include "nautilus-query.h"
-#define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH_HIT
-#include "nautilus-debug.h"
 
 struct _NautilusSearchHit
 {
@@ -149,8 +148,8 @@ nautilus_search_hit_compute_scores (NautilusSearchHit *hit,
     }
 
     hit->relevance = recent_bonus + proximity_bonus + match_bonus;
-    DEBUG ("Hit %s computed relevance %.2f (%.2f + %.2f + %.2f)", hit->uri, hit->relevance,
-           proximity_bonus, recent_bonus, match_bonus);
+    g_debug ("Hit %s computed relevance %.2f (%.2f + %.2f + %.2f)", hit->uri, hit->relevance,
+             proximity_bonus, recent_bonus, match_bonus);
 }
 
 const char *
