@@ -290,13 +290,13 @@ toggle_calendar_icon_clicked (GtkEntry              *entry,
     {
         child = "date-calendar";
         icon_name = "view-list-symbolic";
-        tooltip = _("Show a list to select the date");
+        tooltip = _("Show Time Ranges");
     }
     else
     {
         child = "date-list";
         icon_name = "x-office-calendar-symbolic";
-        tooltip = _("Show a calendar to select the date");
+        tooltip = _("Use Calendar");
     }
 
     gtk_stack_set_visible_child_name (GTK_STACK (popover->around_stack), child);
@@ -495,12 +495,10 @@ static void
 fill_types_listbox (NautilusSearchPopover *popover)
 {
     GtkWidget *row;
-    int i;
-    gint n_groups;
+    guint n_groups = nautilus_mime_types_get_number_of_groups ();
 
-    n_groups = nautilus_mime_types_get_number_of_groups ();
     /* Mimetypes */
-    for (i = 0; i < n_groups; i++)
+    for (guint i = 0; i < n_groups; i++)
     {
         /* On the third row, which is right below "Folders", there should be an
          * separator to logically group the types.
@@ -628,11 +626,13 @@ show_other_types_dialog (NautilusSearchPopover *popover)
     GtkWidget *listview;
     GtkRoot *toplevel;
 
+    gtk_popover_popdown (GTK_POPOVER (popover));
+
     string_model = gtk_string_list_new (NULL);
     mime_infos = g_content_types_get_registered ();
     for (GList *l = mime_infos; l != NULL; l = l->next)
     {
-        gtk_string_list_append (string_model, l->data);
+        gtk_string_list_take (string_model, l->data);
     }
     sorter = gtk_string_sorter_new (gtk_property_expression_new (GTK_TYPE_STRING_OBJECT, NULL, "string"));
     sort_model = gtk_sort_list_model_new (G_LIST_MODEL (string_model), GTK_SORTER (sorter));
@@ -829,6 +829,16 @@ nautilus_search_popover_set_property (GObject      *object,
     }
 }
 
+static void
+nautilus_search_popover_dispose (GObject *obj)
+{
+    NautilusSearchPopover *self = NAUTILUS_SEARCH_POPOVER (obj);
+
+    gtk_popover_set_child (GTK_POPOVER (obj), NULL);
+    gtk_widget_dispose_template (GTK_WIDGET (self), NAUTILUS_TYPE_SEARCH_POPOVER);
+
+    G_OBJECT_CLASS (nautilus_search_popover_parent_class)->dispose (obj);
+}
 
 static void
 nautilus_search_popover_class_init (NautilusSearchPopoverClass *klass)
@@ -839,6 +849,7 @@ nautilus_search_popover_class_init (NautilusSearchPopoverClass *klass)
 
     object_class->get_property = nautilus_search_popover_get_property;
     object_class->set_property = nautilus_search_popover_set_property;
+    object_class->dispose = nautilus_search_popover_dispose;
 
     popover_class->closed = nautilus_search_popover_closed;
 
