@@ -27,12 +27,10 @@
 
 #include "nautilus-application.h"
 #include "nautilus-bookmark.h"
-#include "nautilus-file-undo-manager.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-history-controls.h"
 #include "nautilus-location-entry.h"
 #include "nautilus-pathbar.h"
-#include "nautilus-progress-indicator.h"
 #include "nautilus-view-controls.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-window.h"
@@ -43,31 +41,31 @@ struct _NautilusToolbar
 
     NautilusWindow *window;
 
+    GtkWidget *history_controls_stack;
+    GtkWidget *history_controls;
+    GtkWidget *history_controls_placeholder;
     GtkWidget *path_bar_container;
     GtkWidget *location_entry_container;
     GtkWidget *search_container;
     GtkWidget *toolbar_switcher;
     GtkWidget *path_bar;
     GtkWidget *location_entry;
+    GtkWidget *search_button_stack;
+    GtkWidget *search_button;
+    GtkWidget *search_button_placeholder;
 
     gboolean show_location_entry;
 
-    GtkWidget *app_button;
-    GMenuModel *undo_redo_section;
-
     GtkWidget *sidebar_button;
     gboolean show_sidebar_button;
     gboolean sidebar_button_active;
 
     gboolean show_toolbar_children;
 
-    GtkWidget *search_button;
-
     GtkWidget *location_entry_close_button;
 
     /* active slot & bindings */
     NautilusWindowSlot *window_slot;
-    GBinding *search_binding;
 };
 
 enum
@@ -75,7 +73,6 @@ enum
     PROP_0,
     PROP_SHOW_LOCATION_ENTRY,
     PROP_WINDOW_SLOT,
-    PROP_SEARCHING,
     PROP_SHOW_SIDEBAR_BUTTON,
     PROP_SIDEBAR_BUTTON_ACTIVE,
     PROP_SHOW_TOOLBAR_CHILDREN,
@@ -98,7 +95,7 @@ toolbar_update_appearance (NautilusToolbar *self)
                                                   NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
 
     if (self->window_slot != NULL &&
-        nautilus_window_slot_get_searching (self->window_slot))
+        nautilus_window_slot_get_search_visible (self->window_slot))
     {
         gtk_stack_set_visible_child_name (GTK_STACK (self->toolbar_switcher), "search");
     }
@@ -110,80 +107,14 @@ toolbar_update_appearance (NautilusToolbar *self)
     {
         gtk_stack_set_visible_child_name (GTK_STACK (self->toolbar_switcher), "pathbar");
     }
-}
 
-static void
-update_action (NautilusToolbar *self,
-               const char      *action_name,
-               gboolean         enabled)
-{
-    GtkWidget *window;
-    GAction *action;
-
-    window = gtk_widget_get_ancestor (GTK_WIDGET (self), NAUTILUS_TYPE_WINDOW);
-
-    /* Activate/deactivate */
-    action = g_action_map_lookup_action (G_ACTION_MAP (window), action_name);
-    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-}
-
-static void
-undo_manager_changed (NautilusToolbar *self)
-{
-    NautilusFileUndoInfo *info;
-    NautilusFileUndoManagerState undo_state;
-    gboolean undo_active;
-    gboolean redo_active;
-    g_autofree gchar *undo_label = NULL;
-    g_autofree gchar *redo_label = NULL;
-    g_autofree gchar *undo_description = NULL;
-    g_autofree gchar *redo_description = NULL;
-    gboolean is_undo;
-    g_autoptr (GMenu) updated_section = g_menu_new ();
-    g_autoptr (GMenuItem) undo_menu_item = NULL;
-    g_autoptr (GMenuItem) redo_menu_item = NULL;
-
-    /* Look up the last action from the undo manager, and get the text that
-     * describes it, e.g. "Undo Create Folder"/"Redo Create Folder"
-     */
-    info = nautilus_file_undo_manager_get_action ();
-    undo_state = nautilus_file_undo_manager_get_state ();
-    undo_active = redo_active = FALSE;
-    if (info != NULL && undo_state > NAUTILUS_FILE_UNDO_MANAGER_STATE_NONE)
-    {
-        is_undo = undo_state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO;
-
-        /* The last action can either be undone/redone. Activate the corresponding
-         * menu item and deactivate the other
-         */
-        undo_active = is_undo;
-        redo_active = !is_undo;
-        nautilus_file_undo_info_get_strings (info, &undo_label, &undo_description,
-                                             &redo_label, &redo_description);
-    }
-
-    /* Set the label of the undo and redo menu items, and activate them appropriately
-     */
-    if (!undo_active || undo_label == NULL)
-    {
-        g_free (undo_label);
-        undo_label = g_strdup (_("_Undo"));
-    }
-    undo_menu_item = g_menu_item_new (undo_label, "win.undo");
-    g_menu_append_item (updated_section, undo_menu_item);
-    update_action (self, "undo", undo_active);
-
-    if (!redo_active || redo_label == NULL)
-    {
-        g_free (redo_label);
-        redo_label = g_strdup (_("_Redo"));
-    }
-    redo_menu_item = g_menu_item_new (redo_label, "win.redo");
-    g_menu_append_item (updated_section, redo_menu_item);
-    update_action (self, "redo", redo_active);
-
-    nautilus_gmenu_set_from_model (G_MENU (self->undo_redo_section),
-                                   G_MENU_MODEL (updated_section));
+    /* Adjust to global search mode. */
+    gboolean search_global = (self->window_slot != NULL &&
+                              nautilus_window_slot_get_search_global (self->window_slot));
+    gtk_stack_set_visible_child (GTK_STACK (self->search_button_stack),
+                                 search_global ? self->search_button_placeholder : self->search_button);
+    gtk_stack_set_visible_child (GTK_STACK (self->history_controls_stack),
+                                 search_global ? self->history_controls_placeholder : self->history_controls);
 }
 
 static void
@@ -235,6 +166,7 @@ nautilus_toolbar_constructed (GObject *object)
     gtk_box_append (GTK_BOX (self->location_entry_container),
                     self->location_entry);
     self->location_entry_close_button = gtk_button_new_from_icon_name ("window-close-symbolic");
+    gtk_widget_set_tooltip_text (self->location_entry_close_button, _("Cancel"));
     gtk_box_append (GTK_BOX (self->location_entry_container),
                     self->location_entry_close_button);
     g_signal_connect (self->location_entry_close_button, "clicked",
@@ -256,27 +188,11 @@ static void
 nautilus_toolbar_init (NautilusToolbar *self)
 {
     g_type_ensure (NAUTILUS_TYPE_HISTORY_CONTROLS);
-    g_type_ensure (NAUTILUS_TYPE_PROGRESS_INDICATOR);
     g_type_ensure (NAUTILUS_TYPE_VIEW_CONTROLS);
 
     gtk_widget_init_template (GTK_WIDGET (self));
 }
 
-void
-nautilus_toolbar_on_window_constructed (NautilusToolbar *self)
-{
-    /* undo_manager_changed manipulates the window actions, so set it up
-     * after the window and it's actions have been constructed
-     */
-    g_signal_connect_object (nautilus_file_undo_manager_get (),
-                             "undo-changed",
-                             G_CALLBACK (undo_manager_changed),
-                             self,
-                             G_CONNECT_SWAPPED);
-
-    undo_manager_changed (self);
-}
-
 static void
 nautilus_toolbar_get_property (GObject    *object,
                                guint       property_id,
@@ -299,12 +215,6 @@ nautilus_toolbar_get_property (GObject    *object,
         }
         break;
 
-        case PROP_SEARCHING:
-        {
-            g_value_set_boolean (value, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->search_button)));
-        }
-        break;
-
         case PROP_SHOW_SIDEBAR_BUTTON:
         {
             g_value_set_boolean (value, self->show_sidebar_button);
@@ -339,11 +249,6 @@ on_window_slot_destroyed (gpointer  data,
 
     self = NAUTILUS_TOOLBAR (data);
 
-    /* The window slot was finalized, and the binding has already been removed.
-     * Null it here, so that dispose() does not trip over itself when removing it.
-     */
-    self->search_binding = NULL;
-
     nautilus_toolbar_set_window_slot_real (self, NULL);
 }
 
@@ -369,13 +274,6 @@ nautilus_toolbar_set_property (GObject      *object,
         }
         break;
 
-        case PROP_SEARCHING:
-        {
-            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->search_button),
-                                          g_value_get_boolean (value));
-        }
-        break;
-
         case PROP_SHOW_SIDEBAR_BUTTON:
         {
             self->show_sidebar_button = g_value_get_boolean (value);
@@ -405,11 +303,10 @@ nautilus_toolbar_set_property (GObject      *object,
 static void
 nautilus_toolbar_dispose (GObject *object)
 {
-    NautilusToolbar *self;
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (object);
 
-    self = NAUTILUS_TOOLBAR (object);
-
-    g_clear_pointer (&self->search_binding, g_binding_unbind);
+    adw_bin_set_child (ADW_BIN (object), NULL);
+    gtk_widget_dispose_template (GTK_WIDGET (self), NAUTILUS_TYPE_TOOLBAR);
 
     G_OBJECT_CLASS (nautilus_toolbar_parent_class)->dispose (object);
 }
@@ -462,13 +359,6 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
                              (G_PARAM_READWRITE |
                               G_PARAM_STATIC_STRINGS));
 
-    properties [PROP_SEARCHING] =
-        g_param_spec_boolean ("searching",
-                              "Current view is searching",
-                              "Whether the current view is searching or not",
-                              FALSE,
-                              G_PARAM_READWRITE);
-
     properties[PROP_SHOW_SIDEBAR_BUTTON] =
         g_param_spec_boolean ("show-sidebar-button", NULL, NULL, FALSE,
                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -484,14 +374,16 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
     gtk_widget_class_set_template_from_resource (widget_class,
                                                  "/org/gnome/nautilus/ui/nautilus-toolbar.ui");
 
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, app_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, undo_redo_section);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, history_controls_stack);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, history_controls);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, history_controls_placeholder);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, toolbar_switcher);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_container);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, path_bar_container);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, location_entry_container);
-
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button_stack);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button_placeholder);
 
     gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_TOOLBAR);
 }
@@ -579,15 +471,13 @@ nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
                            on_window_slot_destroyed,
                            self);
 
-        self->search_binding = g_object_bind_property (self->window_slot, "searching",
-                                                       self, "searching",
-                                                       G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-
         g_signal_connect_swapped (self->window_slot, "notify::extensions-background-menu",
                                   G_CALLBACK (slot_on_extensions_background_menu_changed), self);
         g_signal_connect_swapped (self->window_slot, "notify::templates-menu",
                                   G_CALLBACK (slot_on_templates_menu_changed), self);
-        g_signal_connect_swapped (self->window_slot, "notify::searching",
+        g_signal_connect_swapped (self->window_slot, "notify::search-visible",
+                                  G_CALLBACK (toolbar_update_appearance), self);
+        g_signal_connect_swapped (self->window_slot, "notify::search-global",
                                   G_CALLBACK (toolbar_update_appearance), self);
     }
 
@@ -602,7 +492,6 @@ nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
     toolbar_update_appearance (self);
 
     g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_WINDOW_SLOT]);
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SEARCHING]);
 }
 
 void
@@ -617,8 +506,6 @@ nautilus_toolbar_set_window_slot (NautilusToolbar    *self,
         return;
     }
 
-    g_clear_pointer (&self->search_binding, g_binding_unbind);
-
     if (self->window_slot != NULL)
     {
         g_signal_handlers_disconnect_by_data (self->window_slot, self);
@@ -628,16 +515,3 @@ nautilus_toolbar_set_window_slot (NautilusToolbar    *self,
 
     nautilus_toolbar_set_window_slot_real (self, window_slot);
 }
-
-gboolean
-nautilus_toolbar_is_menu_visible (NautilusToolbar *self)
-{
-    GtkWidget *menu;
-
-    g_return_val_if_fail (NAUTILUS_IS_TOOLBAR (self), FALSE);
-
-    menu = GTK_WIDGET (gtk_menu_button_get_popover (GTK_MENU_BUTTON (self->app_button)));
-    g_return_val_if_fail (menu != NULL, FALSE);
-
-    return gtk_widget_is_visible (menu);
-}
