@@ -23,8 +23,8 @@
 
 #pragma once
 
+#include <adwaita.h>
 #include <gtk/gtk.h>
-#include <libadwaita-1/adwaita.h>
 
 #include "nautilus-window-slot.h"
 
@@ -45,10 +45,6 @@ void       nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
 void       nautilus_toolbar_set_active_slot    (NautilusToolbar    *toolbar,
                                                 NautilusWindowSlot *slot);
 
-gboolean   nautilus_toolbar_is_menu_visible    (NautilusToolbar *toolbar);
-
-void       nautilus_toolbar_on_window_constructed       (NautilusToolbar *toolbar);
-
 void nautilus_toolbar_set_window_slot (NautilusToolbar    *self,
                                        NautilusWindowSlot *window_slot);
 G_END_DECLS
