@@ -15,6 +15,7 @@ struct _NautilusViewModel
     GtkMultiSelection *selection_model;
 
     gboolean expand_as_a_tree;
+    GList *cut_files;
 };
 
 static inline GListStore *
@@ -88,6 +89,22 @@ nautilus_view_model_list_model_init (GListModelInterface *iface)
     iface->get_item = nautilus_view_model_get_item;
 }
 
+static void
+nautilus_view_model_get_section (GtkSectionModel *model,
+                                 guint            position,
+                                 guint           *out_start,
+                                 guint           *out_end)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+
+    gtk_section_model_get_section (GTK_SECTION_MODEL (self->sort_model), position, out_start, out_end);
+}
+
+static void
+nautilus_view_model_section_model_init (GtkSectionModelInterface *iface)
+{
+    iface->get_section = nautilus_view_model_get_section;
+}
 
 static gboolean
 nautilus_view_model_is_selected (GtkSelectionModel *model,
@@ -136,6 +153,8 @@ nautilus_view_model_selection_model_init (GtkSelectionModelInterface *iface)
 G_DEFINE_TYPE_WITH_CODE (NautilusViewModel, nautilus_view_model, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
                                                 nautilus_view_model_list_model_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SECTION_MODEL,
+                                                nautilus_view_model_section_model_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL,
                                                 nautilus_view_model_selection_model_init))
 
@@ -167,6 +186,9 @@ dispose (GObject *object)
         g_signal_handlers_disconnect_by_func (self->sort_model,
                                               g_list_model_items_changed,
                                               self);
+        g_signal_handlers_disconnect_by_func (self->sort_model,
+                                              gtk_section_model_sections_changed,
+                                              self);
         g_object_unref (self->sort_model);
         self->sort_model = NULL;
     }
@@ -185,6 +207,8 @@ finalize (GObject *object)
 
     g_hash_table_destroy (self->map_files_to_model);
     g_hash_table_destroy (self->directory_reverse_map);
+
+    g_clear_list (&self->cut_files, g_object_unref);
 }
 
 static void
@@ -240,11 +264,6 @@ create_model_func (GObject           *item,
     NautilusFile *file;
     GListStore *store;
 
-    if (!self->expand_as_a_tree)
-    {
-        return NULL;
-    }
-
     file = nautilus_view_item_get_file (NAUTILUS_VIEW_ITEM (item));
     if (!nautilus_file_is_directory (file))
     {
@@ -280,6 +299,8 @@ constructed (GObject *object)
 
     g_signal_connect_swapped (self->sort_model, "items-changed",
                               G_CALLBACK (g_list_model_items_changed), self);
+    g_signal_connect_swapped (self->sort_model, "sections-changed",
+                              G_CALLBACK (gtk_section_model_sections_changed), self);
     g_signal_connect_swapped (self->selection_model, "selection-changed",
                               G_CALLBACK (gtk_selection_model_selection_changed), self);
 }
@@ -357,66 +378,96 @@ nautilus_view_model_set_sorter (NautilusViewModel *self,
 void
 nautilus_view_model_sort (NautilusViewModel *self)
 {
-    g_autoptr (GtkSorter) sorter = NULL;
+    GtkSorter *sorter = nautilus_view_model_get_sorter (self);
 
-    /* We are not supposed to call gtk_sorter_changed() from here, so let's
-     * re-set the sorter to trigger re-sorting of the list. Hold a reference
-     * to keep the sorter from getting destroyed while re-setting. */
-    sorter = g_object_ref (gtk_sort_list_model_get_sorter (self->sort_model));
-    gtk_sort_list_model_set_sorter (self->sort_model, sorter);
+    if (sorter != NULL)
+    {
+        gtk_sorter_changed (sorter, GTK_SORTER_CHANGE_DIFFERENT);
+    }
 }
 
-GQueue *
-nautilus_view_model_get_items_from_files (NautilusViewModel *self,
-                                          GQueue            *files)
+GList *
+nautilus_view_model_get_sorted_items_for_files (NautilusViewModel *self,
+                                                GList             *files)
 {
-    GList *l;
-    GQueue *items;
+    GList *items = NULL;
 
-    items = g_queue_new ();
-    for (l = g_queue_peek_head_link (files); l != NULL; l = l->next)
+    for (GList *l = files; l != NULL; l = l->next)
     {
         NautilusViewItem *item;
 
-        item = nautilus_view_model_get_item_from_file (self, l->data);
+        item = nautilus_view_model_get_item_for_file (self, l->data);
         if (item != NULL)
         {
-            g_queue_push_tail (items, item);
+            items = g_list_prepend (items, item);
         }
     }
 
-    return items;
+    return g_list_sort_with_data (g_list_copy (items), compare_data_func, self);
 }
 
 NautilusViewItem *
-nautilus_view_model_get_item_from_file (NautilusViewModel *self,
-                                        NautilusFile      *file)
+nautilus_view_model_get_item_for_file (NautilusViewModel *self,
+                                       NautilusFile      *file)
 {
     return g_hash_table_lookup (self->map_files_to_model, file);
 }
 
 void
-nautilus_view_model_remove_item (NautilusViewModel *self,
-                                 NautilusViewItem  *item,
-                                 NautilusDirectory *directory)
+nautilus_view_model_remove_items (NautilusViewModel *self,
+                                  GList             *items,
+                                  NautilusDirectory *directory)
 {
-    NautilusFile *file;
-    g_autoptr (NautilusFile) parent = NULL;
-    GListStore *dir_store;
-    guint i;
+    g_autoptr (NautilusFile) parent = nautilus_directory_get_corresponding_file (directory);
+    GListStore *dir_store = get_directory_store (self, parent);
+    guint new_start, current_start;
+    guint n_items_in_range = 0;
+    g_autoptr (GtkBitset) positions = gtk_bitset_new_empty ();
+    GtkBitsetIter position_iter;
 
-    file = nautilus_view_item_get_file (item);
-    parent = nautilus_directory_get_corresponding_file (directory);
-    dir_store = get_directory_store (self, parent);
-    if (g_list_store_find (dir_store, item, &i))
+    for (GList *l = items; l != NULL; l = l->next)
     {
-        g_list_store_remove (dir_store, i);
+        NautilusViewItem *item = l->data;
+        NautilusFile *file = nautilus_view_item_get_file (item);
+        guint i;
+
+        if (!g_list_store_find (dir_store, item, &i))
+        {
+            g_autofree char *uri = nautilus_file_get_uri (file);
+
+            g_warning ("Failed to remove item %s", uri);
+            continue;
+        }
+
+        gtk_bitset_add (positions, i);
         g_hash_table_remove (self->map_files_to_model, file);
         if (nautilus_file_is_directory (file))
         {
             g_hash_table_remove (self->directory_reverse_map, file);
         }
     }
+
+    /* Remove contiguous item ranges to minimize ::items-changed emissions.
+     * Remove starting from the end, not to impact the index */
+    gtk_bitset_iter_init_last (&position_iter, positions, &new_start);
+    while (gtk_bitset_iter_is_valid (&position_iter))
+    {
+        /* The start of the range is a moving target that gets decremented as we
+         * move up the list. */
+        current_start = new_start;
+        n_items_in_range++;
+
+        if (gtk_bitset_iter_previous (&position_iter, &new_start) &&
+            new_start == current_start - 1)
+        {
+            /* The previous item is contiguous, keep growing the range. */
+            continue;
+        }
+
+        /* Flush full-grown range. */
+        g_list_store_splice (dir_store, current_start, n_items_in_range, 0, 0);
+        n_items_in_range = 0;
+    }
 }
 
 void
@@ -456,16 +507,18 @@ splice_items_into_common_parent (NautilusViewModel *self,
 
 void
 nautilus_view_model_add_items (NautilusViewModel *self,
-                               GQueue            *items)
+                               GList             *items)
 {
     g_autoptr (GPtrArray) array = g_ptr_array_new ();
     g_autoptr (NautilusFile) previous_parent = NULL;
-    GList *l;
+    g_autoptr (GList) sorted_items = NULL;
     NautilusViewItem *item;
 
-    g_queue_sort (items, compare_data_func, self);
+    /* The first added file becomes the initial focus and scroll anchor, so we
+     * need to sort items before adding them to the internal model. */
+    sorted_items = g_list_sort_with_data (g_list_copy (items), compare_data_func, self);
 
-    for (l = g_queue_peek_head_link (items); l != NULL; l = l->next)
+    for (GList *l = sorted_items; l != NULL; l = l->next)
     {
         g_autoptr (NautilusFile) parent = NULL;
 
@@ -496,35 +549,6 @@ nautilus_view_model_add_items (NautilusViewModel *self,
     }
 }
 
-guint
-nautilus_view_model_get_index (NautilusViewModel *self,
-                               NautilusViewItem  *item)
-{
-    guint n_items;
-    guint i = 0;
-
-    n_items = g_list_model_get_n_items (G_LIST_MODEL (self->sort_model));
-    while (i < n_items)
-    {
-        g_autoptr (GtkTreeListRow) row = NULL;
-        g_autoptr (NautilusViewItem) item_i = NULL;
-
-        row = g_list_model_get_item (G_LIST_MODEL (self->sort_model), i);
-        g_warn_if_fail (GTK_IS_TREE_LIST_ROW (row));
-
-        item_i = gtk_tree_list_row_get_item (row);
-        g_warn_if_fail (NAUTILUS_IS_VIEW_ITEM (item_i));
-
-        if (item_i == item)
-        {
-            return i;
-        }
-        i++;
-    }
-
-    return G_MAXUINT;
-}
-
 void
 nautilus_view_model_clear_subdirectory (NautilusViewModel *self,
                                         NautilusViewItem  *item)
@@ -552,9 +576,55 @@ nautilus_view_model_clear_subdirectory (NautilusViewModel *self,
     g_hash_table_remove (self->directory_reverse_map, file);
 }
 
+static inline void
+collapse_all_rows (NautilusViewModel *self)
+{
+    guint n_root_items = g_list_model_get_n_items (gtk_tree_list_model_get_model (self->tree_model));
+
+    for (guint i = 0; i < n_root_items; i++)
+    {
+        g_autoptr (GtkTreeListRow) root_level_row = gtk_tree_list_model_get_child_row (self->tree_model, i);
+
+        gtk_tree_list_row_set_expanded (root_level_row, FALSE);
+    }
+}
+
 void
 nautilus_view_model_expand_as_a_tree (NautilusViewModel *self,
                                       gboolean           expand_as_a_tree)
 {
+    if (self->expand_as_a_tree && !expand_as_a_tree)
+    {
+        collapse_all_rows (self);
+    }
+
     self->expand_as_a_tree = expand_as_a_tree;
 }
+
+void
+nautilus_view_model_set_cut_files (NautilusViewModel *self,
+                                   GList             *cut_files)
+{
+    NautilusViewItem *item;
+
+    for (GList *l = self->cut_files; l != NULL; l = l->next)
+    {
+        item = nautilus_view_model_get_item_for_file (self, l->data);
+        if (item != NULL)
+        {
+            nautilus_view_item_set_cut (item, FALSE);
+        }
+    }
+    g_clear_list (&self->cut_files, g_object_unref);
+
+    for (GList *l = cut_files; l != NULL; l = l->next)
+    {
+        item = nautilus_view_model_get_item_for_file (self, l->data);
+        if (item != NULL)
+        {
+            self->cut_files = g_list_prepend (self->cut_files,
+                                              g_object_ref (l->data));
+            nautilus_view_item_set_cut (item, TRUE);
+        }
+    }
+}
