@@ -23,6 +23,7 @@
  */
 
 /* nautilus-window.c: Implementation of the main window object */
+#define G_LOG_DOMAIN "nautilus-window"
 
 #include "nautilus-window.h"
 
@@ -41,9 +42,6 @@
 #include <gdk/x11/gdkx.h>
 #endif
 
-#define DEBUG_FLAG NAUTILUS_DEBUG_WINDOW
-#include "nautilus-debug.h"
-
 #include "gtk/nautilusgtkplacessidebarprivate.h"
 
 #include "nautilus-application.h"
@@ -60,7 +58,6 @@
 #include "nautilus-mime-actions.h"
 #include "nautilus-module.h"
 #include "nautilus-pathbar.h"
-#include "nautilus-profile.h"
 #include "nautilus-progress-indicator.h"
 #include "nautilus-scheme.h"
 #include "nautilus-signaller.h"
@@ -71,8 +68,8 @@
 
 /* Forward and back buttons on the mouse */
 static gboolean mouse_extra_buttons = TRUE;
-static int mouse_forward_button = 9;
-static int mouse_back_button = 8;
+static guint mouse_forward_button = 9;
+static guint mouse_back_button = 8;
 
 static void mouse_back_button_changed (gpointer callback_data);
 static void mouse_forward_button_changed (gpointer callback_data);
@@ -266,24 +263,24 @@ action_up (GSimpleAction *action,
            GVariant      *state,
            gpointer       user_data)
 {
-    NautilusWindowSlot *slot;
-    GFile *parent, *location;
+    NautilusWindowSlot *slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
 
-    slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
-    location = nautilus_window_slot_get_location (slot);
-
-    if (location != NULL)
+    if (slot != NULL)
     {
-        parent = g_file_get_parent (location);
-        if (parent != NULL)
-        {
-            nautilus_window_open_location_full (NAUTILUS_WINDOW (user_data),
-                                                parent,
-                                                0,
-                                                NULL, NULL);
-        }
+        nautilus_window_slot_go_up (slot);
+    }
+}
 
-        g_clear_object (&parent);
+static void
+action_down (GSimpleAction *action,
+             GVariant      *state,
+             gpointer       user_data)
+{
+    NautilusWindowSlot *slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
+
+    if (slot != NULL)
+    {
+        nautilus_window_slot_go_down (slot);
     }
 }
 
@@ -486,6 +483,30 @@ on_slot_location_changed (NautilusWindowSlot *slot,
     }
 }
 
+static void
+on_slot_search_global_changed (NautilusWindowSlot *slot,
+                               GParamSpec         *pspec,
+                               NautilusWindow     *self)
+{
+    NautilusGtkPlacesSidebar *sidebar = NAUTILUS_GTK_PLACES_SIDEBAR (self->places_sidebar);
+
+    if (nautilus_window_get_active_slot (self) != slot)
+    {
+        return;
+    }
+
+    if (nautilus_window_slot_get_search_global (slot))
+    {
+        nautilus_gtk_places_sidebar_set_location (sidebar, NULL);
+    }
+    else
+    {
+        nautilus_gtk_places_sidebar_set_location (sidebar, nautilus_window_slot_get_location (slot));
+    }
+
+    nautilus_window_sync_location_widgets (self);
+}
+
 static void
 tab_view_setup_menu_cb (AdwTabView     *tab_view,
                         AdwTabPage     *page,
@@ -549,6 +570,8 @@ connect_slot (NautilusWindow     *window,
 {
     g_signal_connect (slot, "notify::location",
                       G_CALLBACK (on_slot_location_changed), window);
+    g_signal_connect (slot, "notify::search-global",
+                      G_CALLBACK (on_slot_search_global_changed), window);
 }
 
 static void
@@ -570,17 +593,17 @@ nautilus_window_create_and_init_slot (NautilusWindow    *window,
     return slot;
 }
 
-static gboolean
-location_to_tooltip (GBinding           *binding,
-                     const GValue       *input,
-                     GValue             *output,
-                     NautilusWindowSlot *slot)
+static void
+on_update_page_tooltip (NautilusWindowSlot *slot,
+                        GParamSpec         *pspec,
+                        AdwTabPage         *page)
 {
-    GFile *location = g_value_get_object (input);
+    GFile *location = nautilus_window_slot_get_location (slot);
+    g_autofree char *escaped_name = NULL;
 
     if (location == NULL)
     {
-        return TRUE;
+        return;
     }
 
     /* Set the tooltip on the label's parent (the tab label hbox),
@@ -589,15 +612,15 @@ location_to_tooltip (GBinding           *binding,
 
     if (g_file_has_uri_scheme (location, SCHEME_SEARCH))
     {
-        g_value_set_string (output, nautilus_window_slot_get_title (slot));
+        escaped_name = g_markup_escape_text (nautilus_window_slot_get_title (slot), -1);
     }
     else
     {
         g_autofree gchar *location_name = g_file_get_parse_name (location);
-        g_value_set_string (output, location_name);
+        escaped_name = g_markup_escape_text (location_name, -1);
     }
 
-    return TRUE;
+    adw_tab_page_set_tooltip (page, escaped_name);
 }
 
 void
@@ -621,11 +644,7 @@ nautilus_window_initialize_slot (NautilusWindow     *window,
     g_object_bind_property (slot, "title",
                             page, "title",
                             G_BINDING_SYNC_CREATE);
-    g_object_bind_property_full (slot, "location",
-                                 page, "tooltip",
-                                 G_BINDING_SYNC_CREATE,
-                                 (GBindingTransformFunc) location_to_tooltip,
-                                 NULL, slot, NULL);
+    g_signal_connect (slot, "notify::title", G_CALLBACK (on_update_page_tooltip), page);
 }
 
 void
@@ -652,7 +671,7 @@ nautilus_window_open_location_full (NautilusWindow     *window,
     }
 
     /* Make the opened location the one active if we weren't ask for the
-     * oposite, since it's the most usual use case */
+     * opposite, since it's the most usual use case */
     if (!(flags & NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE))
     {
         gtk_window_present (GTK_WINDOW (window));
@@ -742,7 +761,7 @@ remove_slot_from_window (NautilusWindowSlot *slot,
     g_return_if_fail (NAUTILUS_IS_WINDOW_SLOT (slot));
     g_return_if_fail (NAUTILUS_WINDOW (window));
 
-    DEBUG ("Removing slot %p", slot);
+    g_debug ("Removing slot %p", slot);
 
     disconnect_slot (window, slot);
     window->slots = g_list_remove (window->slots, slot);
@@ -851,6 +870,7 @@ open_location_cb (NautilusWindow             *window,
 {
     NautilusOpenFlags flags;
     NautilusApplication *application;
+    AdwOverlaySplitView *split_view = ADW_OVERLAY_SPLIT_VIEW (window->split_view);
 
     switch (open_flags)
     {
@@ -881,6 +901,12 @@ open_location_cb (NautilusWindow             *window,
      */
     nautilus_application_open_location_full (application, location, flags,
                                              NULL, window, NULL);
+
+    if (adw_overlay_split_view_get_collapsed (split_view) &&
+        open_flags == NAUTILUS_GTK_PLACES_OPEN_NORMAL)
+    {
+        adw_overlay_split_view_set_show_sidebar (split_view, FALSE);
+    }
 }
 
 /* Callback used when the places sidebar needs us to present an error message */
@@ -1065,7 +1091,7 @@ nautilus_window_slot_close (NautilusWindow     *window,
     NautilusNavigationState *data;
     AdwTabPage *page;
 
-    DEBUG ("Requesting to remove slot %p from window %p", slot, window);
+    g_debug ("Requesting to remove slot %p from window %p", slot, window);
     if (window == NULL || slot == NULL)
     {
         return;
@@ -1086,7 +1112,7 @@ nautilus_window_slot_close (NautilusWindow     *window,
     /* If that was the last slot in the window, close the window. */
     if (window->slots == NULL)
     {
-        DEBUG ("Last slot removed, closing the window");
+        g_debug ("Last slot removed, closing the window");
         nautilus_window_close (window);
     }
 }
@@ -1141,13 +1167,15 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
         nautilus_path_bar_set_path (NAUTILUS_PATH_BAR (path_bar), location);
     }
 
-    enabled = nautilus_window_slot_get_back_history (slot) != NULL;
+    enabled = (nautilus_window_slot_get_back_history (slot) != NULL &&
+               !nautilus_window_slot_get_search_global (slot));
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "back");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "back-n");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 
-    enabled = nautilus_window_slot_get_forward_history (slot) != NULL;
+    enabled = (nautilus_window_slot_get_forward_history (slot) != NULL &&
+               !nautilus_window_slot_get_search_global (slot));
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward-n");
@@ -1174,12 +1202,11 @@ nautilus_window_ensure_location_entry (NautilusWindow *window)
 static gchar *
 toast_undo_deleted_get_label (NautilusFileUndoInfo *undo_info)
 {
-    GList *files;
+    g_autoptr (GList) files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
     gchar *file_label;
     gchar *label;
     gint length;
 
-    files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
     length = g_list_length (files);
     if (length == 1)
     {
@@ -1209,9 +1236,7 @@ toast_undo_unstar_get_label (NautilusFileUndoInfo *undo_info)
     length = g_list_length (files);
     if (length == 1)
     {
-        g_autofree gchar *file_label = NULL;
-
-        file_label = nautilus_file_get_display_name (files->data);
+        const char *file_label = nautilus_file_get_display_name (files->data);
         /* Translators: one item has been unstarred and %s is its name. */
         label = g_markup_printf_escaped (_("“%s” unstarred"), file_label);
     }
@@ -1359,16 +1384,21 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
 void
 nautilus_window_show_operation_notification (NautilusWindow *window,
                                              gchar          *main_label,
-                                             GFile          *folder_to_open)
+                                             GFile          *folder_to_open,
+                                             gboolean        was_quick)
 {
-    gchar *button_label;
-    gchar *folder_name;
-    NautilusFile *folder;
-    GVariant *target;
-    GFile *current_location;
+    gboolean is_current_location;
     AdwToast *toast;
 
-    if (window->active_slot == NULL)
+    if (window->active_slot == NULL || !gtk_window_is_active (GTK_WINDOW (window)))
+    {
+        return;
+    }
+
+    is_current_location = g_file_equal (folder_to_open,
+                                        nautilus_window_slot_get_location (window->active_slot));
+
+    if (is_current_location && was_quick)
     {
         return;
     }
@@ -1377,25 +1407,23 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
     adw_toast_set_priority (toast, ADW_TOAST_PRIORITY_HIGH);
     adw_toast_set_use_markup (toast, FALSE);
 
-    current_location = nautilus_window_slot_get_location (window->active_slot);
-    if (gtk_window_is_active (GTK_WINDOW (window)))
+    if (!is_current_location)
     {
-        if (!g_file_equal (folder_to_open, current_location))
-        {
-            target = g_variant_new_take_string (g_file_get_uri (folder_to_open));
-            folder = nautilus_file_get (folder_to_open);
-            folder_name = nautilus_file_get_display_name (folder);
-            button_label = g_strdup_printf (_("Open %s"), folder_name);
-            adw_toast_set_button_label (toast, button_label);
-            adw_toast_set_action_name (toast, "win.open-location");
-            adw_toast_set_action_target_value (toast, target);
-            nautilus_file_unref (folder);
-            g_free (folder_name);
-            g_free (button_label);
-        }
+        g_autoptr (NautilusFile) folder = NULL;
+        g_autoptr (GString) button_label = g_string_new ("");
+        GVariant *target;
 
-        adw_toast_overlay_add_toast (window->toast_overlay, toast);
+        target = g_variant_new_take_string (g_file_get_uri (folder_to_open));
+        folder = nautilus_file_get (folder_to_open);
+        g_string_printf (button_label, _("Open %s"), nautilus_file_get_display_name (folder));
+        /* Need to escape mnemonics since it's a button. */
+        g_string_replace (button_label, "_", "__", -1);
+        adw_toast_set_button_label (toast, button_label->str);
+        adw_toast_set_action_name (toast, "win.open-location");
+        adw_toast_set_action_target_value (toast, target);
     }
+
+    adw_toast_overlay_add_toast (window->toast_overlay, toast);
 }
 
 static void
@@ -1576,33 +1604,34 @@ extra_drag_drop_cb (AdwTabBar    *self,
 
 const GActionEntry win_entries[] =
 {
-    { "back", action_back },
-    { "forward", action_forward },
-    { "back-n", action_back_n, "u" },
-    { "forward-n", action_forward_n, "u" },
-    { "up", action_up },
-    { "current-location-menu", action_show_current_location_menu },
-    { "open-location", action_open_location, "s" },
-    { "reload", action_reload },
-    { "stop", action_stop },
-    { "new-tab", action_new_tab },
-    { "enter-location", action_enter_location },
-    { "bookmark-current-location", action_bookmark_current_location },
-    { "undo", action_undo },
-    { "redo", action_redo },
-    /* Only accesible by shorcuts */
-    { "close-current-view", action_close_current_view },
-    { "close-other-tabs", action_close_other_tabs },
-    { "go-home", action_go_home },
-    { "go-starred", action_go_starred },
-    { "tab-move-left", action_tab_move_left },
-    { "tab-move-right", action_tab_move_right },
-    { "tab-move-new-window", action_tab_move_new_window },
-    { "prompt-root-location", action_prompt_for_location_root },
-    { "prompt-home-location", action_prompt_for_location_home },
-    { "go-to-tab", NULL, "i", "0", action_go_to_tab },
-    { "restore-tab", action_restore_tab },
-    { "toggle-sidebar", action_toggle_sidebar },
+    { .name = "back", .activate = action_back },
+    { .name = "forward", .activate = action_forward },
+    { .name = "back-n", .activate = action_back_n, .parameter_type = "u" },
+    { .name = "forward-n", .activate = action_forward_n, .parameter_type = "u" },
+    { .name = "up", .activate = action_up },
+    { .name = "down", .activate = action_down },
+    { .name = "current-location-menu", .activate = action_show_current_location_menu },
+    { .name = "open-location", .activate = action_open_location, .parameter_type = "s" },
+    { .name = "reload", .activate = action_reload },
+    { .name = "stop", .activate = action_stop },
+    { .name = "new-tab", .activate = action_new_tab },
+    { .name = "enter-location", .activate = action_enter_location },
+    { .name = "bookmark-current-location", .activate = action_bookmark_current_location },
+    { .name = "undo", .activate = action_undo },
+    { .name = "redo", .activate = action_redo },
+    /* Only accessible by shorcuts */
+    { .name = "close-current-view", .activate = action_close_current_view },
+    { .name = "close-other-tabs", .activate = action_close_other_tabs },
+    { .name = "go-home", .activate = action_go_home },
+    { .name = "go-starred", .activate = action_go_starred },
+    { .name = "tab-move-left", .activate = action_tab_move_left },
+    { .name = "tab-move-right", .activate = action_tab_move_right },
+    { .name = "tab-move-new-window", .activate = action_tab_move_new_window },
+    { .name = "prompt-root-location", .activate = action_prompt_for_location_root },
+    { .name = "prompt-home-location", .activate = action_prompt_for_location_home },
+    { .name = "go-to-tab", .parameter_type = "i", .state = "0", .change_state = action_go_to_tab },
+    { .name = "restore-tab", .activate = action_restore_tab },
+    { .name = "toggle-sidebar", .activate = action_toggle_sidebar },
 };
 
 static void
@@ -1642,9 +1671,10 @@ nautilus_window_initialize_actions (NautilusWindow *window)
 
     nautilus_application_set_accelerator (app, "win.undo", "<control>z");
     nautilus_application_set_accelerator (app, "win.redo", "<shift><control>z");
-    /* Only accesible by shorcuts */
+    /* Only accessible by shorcuts */
     nautilus_application_set_accelerators (app, "win.bookmark-current-location", ACCELS ("<control>d", "AddFavorite"));
     nautilus_application_set_accelerator (app, "win.up", "<alt>Up");
+    nautilus_application_set_accelerator (app, "win.down", "<alt>Down");
     nautilus_application_set_accelerators (app, "win.go-home", ACCELS ("<alt>Home", "HomePage", "Start"));
     nautilus_application_set_accelerator (app, "win.go-starred", "Favorites");
     nautilus_application_set_accelerator (app, "win.tab-move-left", "<shift><control>Page_Up");
@@ -1664,6 +1694,8 @@ nautilus_window_initialize_actions (NautilusWindow *window)
         nautilus_application_set_accelerator (app, detailed_action, accel);
     }
 
+    nautilus_window_on_undo_changed (nautilus_file_undo_manager_get (), window);
+
 #undef ACCELS
 
     action = g_action_map_lookup_action (G_ACTION_MAP (window), "toggle-sidebar");
@@ -1671,17 +1703,53 @@ nautilus_window_initialize_actions (NautilusWindow *window)
                             action, "enabled", G_BINDING_SYNC_CREATE);
 }
 
+static gboolean
+is_layout_reversed (void)
+{
+    GtkSettings *settings = gtk_settings_get_default ();
+    g_autofree char *layout = NULL;
+    g_auto (GStrv) parts = NULL;
+
+    if (settings != NULL)
+    {
+        g_object_get (settings, "gtk-decoration-layout", &layout, NULL);
+    }
+
+    if (layout != NULL)
+    {
+        parts = g_strsplit (layout, ":", 2);
+    }
+
+    /* Invalid layout, don't even try */
+    if (parts == NULL || g_strv_length (parts) < 2)
+    {
+        return FALSE;
+    }
+
+    g_return_val_if_fail (parts[0] != NULL, FALSE);
+    return (g_strrstr (parts[0], "close") != NULL);
+}
+
+static void
+notify_decoration_layout_cb (NautilusWindow *self)
+{
+    gboolean inverted = is_layout_reversed ();
+
+    if (ADW_IS_TAB_BAR (self->tab_bar))
+    {
+        adw_tab_bar_set_inverted (self->tab_bar, inverted);
+    }
+}
 
 static void
 nautilus_window_constructed (GObject *self)
 {
     NautilusWindow *window;
     NautilusApplication *application;
+    GtkSettings *settings;
 
     window = NAUTILUS_WINDOW (self);
 
-    nautilus_profile_start (NULL);
-
     G_OBJECT_CLASS (nautilus_window_parent_class)->constructed (self);
 
     application = NAUTILUS_APPLICATION (g_application_get_default ());
@@ -1712,6 +1780,12 @@ nautilus_window_constructed (GObject *self)
     }
 #endif
 
+    settings = gtk_settings_get_default ();
+    g_signal_connect_object (settings, "notify::gtk-decoration-layout",
+                             G_CALLBACK (notify_decoration_layout_cb), window,
+                             G_CONNECT_SWAPPED);
+    notify_decoration_layout_cb (window);
+
     nautilus_window_set_up_sidebar (window);
 
 
@@ -1727,8 +1801,6 @@ nautilus_window_constructed (GObject *self)
                                                     "changed",
                                                     G_CALLBACK (nautilus_window_sync_bookmarks),
                                                     window, G_CONNECT_SWAPPED);
-
-    nautilus_profile_end (NULL);
 }
 
 static void
@@ -1741,7 +1813,7 @@ nautilus_window_dispose (GObject *object)
     window = NAUTILUS_WINDOW (object);
     application = gtk_window_get_application (GTK_WINDOW (window));
 
-    DEBUG ("Destroying window");
+    g_debug ("Destroying window");
 
     /* close all slots safely */
     slots_copy = g_list_copy (window->slots);
@@ -1761,6 +1833,8 @@ nautilus_window_dispose (GObject *object)
 
     nautilus_window_unexport_handle (window);
 
+    gtk_widget_dispose_template (GTK_WIDGET (window), NAUTILUS_TYPE_WINDOW);
+
     G_OBJECT_CLASS (nautilus_window_parent_class)->dispose (object);
 }
 
@@ -1849,7 +1923,7 @@ nautilus_window_set_active_slot (NautilusWindow     *window,
         return;
     }
 
-    DEBUG ("Setting new slot %p as active, old slot inactive %p", new_slot, old_slot);
+    g_debug ("Setting new slot %p as active, old slot inactive %p", new_slot, old_slot);
 
     /* make old slot inactive if it exists (may be NULL after init, for example) */
     if (old_slot != NULL)
@@ -2025,9 +2099,11 @@ nautilus_window_unexport_handle (NautilusWindow *window)
     if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (GTK_WIDGET (window))))
     {
         GdkSurface *gdk_surface = gtk_native_get_surface (GTK_NATIVE (window));
-        if (GDK_IS_WAYLAND_TOPLEVEL (gdk_surface))
+        if (GDK_IS_WAYLAND_TOPLEVEL (gdk_surface) &&
+            g_str_has_prefix (window->export_handle, "wayland:"))
         {
-            gdk_wayland_toplevel_unexport_handle (GDK_WAYLAND_TOPLEVEL (gdk_surface));
+            gdk_wayland_toplevel_drop_exported_handle (GDK_WAYLAND_TOPLEVEL (gdk_surface),
+                                                       window->export_handle + strlen ("wayland:"));
         }
     }
 #endif
@@ -2179,7 +2255,7 @@ mouse_back_button_changed (gpointer callback_data)
 {
     int new_back_button;
 
-    new_back_button = g_settings_get_int (nautilus_preferences, NAUTILUS_PREFERENCES_MOUSE_BACK_BUTTON);
+    new_back_button = g_settings_get_uint (nautilus_preferences, NAUTILUS_PREFERENCES_MOUSE_BACK_BUTTON);
 
     /* Bounds checking */
     if (new_back_button < 6 || new_back_button > UPPER_MOUSE_LIMIT)
@@ -2195,7 +2271,7 @@ mouse_forward_button_changed (gpointer callback_data)
 {
     int new_forward_button;
 
-    new_forward_button = g_settings_get_int (nautilus_preferences, NAUTILUS_PREFERENCES_MOUSE_FORWARD_BUTTON);
+    new_forward_button = g_settings_get_uint (nautilus_preferences, NAUTILUS_PREFERENCES_MOUSE_FORWARD_BUTTON);
 
     /* Bounds checking */
     if (new_forward_button < 6 || new_forward_button > UPPER_MOUSE_LIMIT)
@@ -2417,17 +2493,17 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
 
     const gchar *designers[] =
     {
-        "The GNOME Project",
+        _("The GNOME Project"),
         NULL
     };
     const gchar *developers[] =
     {
-        "The contributors to the Nautilus project",
+        _("The GNOME Project"),
         NULL
     };
     const gchar *documenters[] =
     {
-        "GNOME Documentation Team",
+        _("The GNOME Project"),
         "Sun Microsystems",
         NULL
     };
@@ -2449,8 +2525,8 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
                            "application-icon", APPLICATION_ID,
                            "developer-name", _("The GNOME Project"),
                            "version", VERSION,
-                           "website", "https://wiki.gnome.org/action/show/Apps/Files",
-                           "issue-url", "https://gitlab.gnome.org/GNOME/nautilus/-/issues/new",
+                           "website", "https://apps.gnome.org/Nautilus/",
+                           "issue-url", "https://gitlab.gnome.org/GNOME/nautilus/-/issues",
                            "support-url", "https://discourse.gnome.org/tag/nautilus",
                            "debug-info", debug_info,
                            "copyright", "© 1999 The Files Authors",
